@@ -6,11 +6,11 @@ nimbleOrRfunctionNames <- c('[','+','-','/','*','(','exp','log','pow','^','%%','
                             'sqrt', 'logit', 'expit', 'ilogit', 'probit', 'iprobit', 'phi', 'cloglog', 'icloglog', 'step', 'nimStep',
                             'sin','cos','tan','asin','acos','atan','cosh','sinh','tanh', 'asinh', 'acosh', 'atanh',
                             'cube', 'abs', 'lgamma', 'loggam', 'log1p', 'lfactorial', ##'factorial', 'gamma',
-                            'ceiling', 'floor', 'round', 'trunc',
+                            'ceiling', 'floor', 'round', 'nimRound', 'trunc',
                             'mean','sum','sd','var','max','min','prod',
                             'asRow', 'asCol',
-                            'chol', 'inverse', 'forwardsolve', 'backsolve', 'solve',   ## removed these from BUGS functions, pending problems with Eigen
-                            '>', '<', '>=', '<=', '==', '!=', '&', '|',
+                            'chol', 'inverse', 'forwardsolve', 'backsolve', 'solve', 'nimEigen', 'nimSvd',  ## removed these from BUGS functions, pending problems with Eigen
+                            '>', '<', '>=', '<=', '==', '!=', '&', '|', '$',
                             distributionFuns,
                             # these are allowed in DSL as special cases even though exp_nimble and t_nonstandard are the canonical NIMBLE distribution functions
                             paste0(c('d','r','q','p'), 't'),
@@ -337,6 +337,9 @@ getSymbolicParentNodesRecurse <- function(code, constNames = list(), indexNames
     cLength <- length(code)
     if(cLength == 1) {
         if(is.name(code)) {
+            if(code == ''){
+              return(list(code = NULL, replaceable = TRUE, hasIndex = FALSE))
+            }
             if(any(code == indexNames)) {
                 return(list(code = NULL, replaceable = TRUE, hasIndex = TRUE))
             }
@@ -351,7 +354,9 @@ getSymbolicParentNodesRecurse <- function(code, constNames = list(), indexNames
     if(is.call(code)) {
         indexingBracket <- code[[1]] == '['
         if(indexingBracket) {
-            if(is.call(code[[2]])) indexingBracket <- FALSE ## treat like any other function
+            if(is.call(code[[2]])){
+              indexingBracket <- FALSE
+            } 
         }
         if(indexingBracket) { ##if(code[[1]] == '[') {
             contents <- lapply(code[-c(1,2)], function(x) getSymbolicParentNodesRecurse(x, constNames, indexNames, nimbleFunctionNames))
@@ -386,7 +391,10 @@ getSymbolicParentNodesRecurse <- function(code, constNames = list(), indexNames
             }
         } else {
             if(cLength > 1) {
-                contents <- lapply(code[-1], function(x) getSymbolicParentNodesRecurse(x, constNames, indexNames, nimbleFunctionNames))
+                if(code[[1]] == '$')
+                  contents <- lapply(code[2],  function(x) getSymbolicParentNodesRecurse(x, constNames, indexNames, nimbleFunctionNames))
+                else
+                  contents <- lapply(code[-1], function(x) getSymbolicParentNodesRecurse(x, constNames, indexNames, nimbleFunctionNames))
                 contentsCode <- unlist(lapply(contents, function(x) x$code), recursive = FALSE)
                 contentsHasIndex <- unlist(lapply(contents, function(x) x$hasIndex))
                 ## if(code[[1]] == ':') return(list(code = contentsCode, ## need a new part of the list for hasIndexingBlock, or can I set hasIndex = TRUE
@@ -402,14 +410,13 @@ getSymbolicParentNodesRecurse <- function(code, constNames = list(), indexNames
             isRfunction <- !any(code[[1]] == nimbleFunctionNames)
             funName <- deparse(code[[1]])
             isRonly <- isRfunction &
-                !checkNimbleOrRfunctionNames(funName)
+                (!checkNimbleOrRfunctionNames(funName))
             if(isRonly & !allContentsReplaceable) {
                 if(!exists(funName))
                     stop("R function '", funName,"' does not exist.")
                 unreplaceable <- sapply(contents[!contentsReplaceable], function(x) as.character(x$code))
                 stop("R function '", funName,"' has arguments that cannot be evaluated; either the function must be a nimbleFunction or values for the following inputs must be specified as constants in the model: ", paste(unreplaceable, collapse = ","), ".")
             }
-
             return(list(code = contentsCode,
                         replaceable = allContentsReplaceable & isRfunction,
                         hasIndex = any(contentsHasIndex)))
@@ -475,6 +482,7 @@ genReplacementsAndCodeRecurse <- function(code, constAndIndexNames, nimbleFuncti
         if(assignment)         return(replaceWhatWeCan(code, contentsCodeReplaced, contentsReplacements, contentsReplaceable, startingAt=2))
         isRfunction <- !any(code[[1]] == nimbleFunctionNames)
         isRonly <- isRfunction & !checkNimbleOrRfunctionNames(deparse(code[[1]]))
+        if(deparse(code[[1]]) == '$') isRonly <- FALSE
         if(isRonly & !allContentsReplaceable) stop(paste0('Error, R function \"', deparse(code[[1]]),'\" has non-replaceable node values as arguments.  Must be a nimble function.'))
         if(isRfunction & allContentsReplaceable)   return(replaceAllCodeSuccessfully(code))
         return(replaceWhatWeCan(code, contentsCodeReplaced, contentsReplacements, contentsReplaceable, startingAt=2))
