@@ -146,7 +146,9 @@ testBUGSmodel <- function(example = NULL, dir = NULL, model = NULL, data = NULL,
       }
     })
 
-    if(.Platform$OS.type != 'windows') dyn.unload(project$cppProjects[[1]]$getSOName())
+    if(.Platform$OS.type != 'windows')
+        clearCompiled(project)
+        ##dyn.unload(project$cppProjects[[1]]$getSOName())
     # this works to avoid having too many DLLs, but gives segfault when one quits R afterwards
     if(debug) browser()
   } else warning("testBUGSmodel: testthat package is required.")
