@@ -33,16 +33,134 @@ NULL
 #' @export
 dwish_chol <- function(x, cholesky, df, scale_param = TRUE, log = FALSE) {
   # scale_param = TRUE is the GCSR parameterization (i.e., scale matrix); scale_param = FALSE is the BUGS parameterization (i.e., rate matrix)
-  .Call('C_dwish_chol', as.double(x), as.double(cholesky), as.double(df), as.double(scale_param), as.logical(log))
+    if(storage.mode(cholesky) != 'double')
+          storage.mode(cholesky) <- 'double'
+    if(storage.mode(x) != 'double')
+            storage.mode(x) <- 'double'
+    .Call('C_dwish_chol', x, cholesky, as.double(df), as.double(scale_param), as.logical(log))
 }
 
 #' @rdname Wishart
 #' @export
 rwish_chol <- function(n = 1, cholesky, df, scale_param = TRUE) {
     if(n != 1) warning('rwish_chol only handles n = 1 at the moment')
-    matrix(.Call('C_rwish_chol', as.double(cholesky), as.double(df), as.double(scale_param)), nrow = sqrt(length(cholesky)))
+    if(storage.mode(cholesky) != 'double')
+    	storage.mode(cholesky) <- 'double'
+    out <- .Call('C_rwish_chol', cholesky, as.double(df), as.double(scale_param))
+    if(!is.null(out)) out <- matrix(out, nrow = sqrt(length(cholesky)))
+    return(out)
 }
 
+
+#' Spectral Decomposition of a Matrix  
+#'
+#' Computes eigenvalues and eigenvectors of a numeric matrix.  
+#' 
+#' @param x a symmetric numeric matrix (double or integer) whose spectral decomposition is to be computed.
+#' @param only.values if TRUE, only the eigenvalues are computed, otherwise both eigenvalues and eigenvectors are computed; default is \code{FALSE}.
+#'
+#' @aliases eigen
+#'
+#' @author NIMBLE development team
+#'
+#' @export
+#'
+#' @details
+#' Computes the spectral decomposition of a symmetric numeric matrix using the Eigen C++ template library. 
+#' Unlike the \code{eigen} function in base R, \code{nimEigen} only works for symmetric matrices.  In a nimbleFunction, \code{eigen} is identical to \code{nimEigen}. 
+#' 
+#' Additionally, \code{returnType(eigenNimbleList())} can be used within a \code{link{nimbleFunction}} to specify that the function will return a \code{\link{nimbleList}} generated by the \code{nimEigen} function.  \code{eigenNimbleList()} can also be used to define a nested \code{\link{nimbleList}} element.  See the User Manual for usage examples. 
+#' 
+#' @return
+#' The spectral decomposition of \code{x} is returned as a \code{\link{nimbleList}} with elements:
+#' \itemize{
+#' \item values vector containing the eigenvalues of \code{x}, sorted in decreasing order.  Since \code{x} is required to be symmetric, all eigenvalues will be real numbers.
+#' \item vectors. matrix with columns containing the eigenvectors of \code{x}, or an empty matrix if \code{only.values} is \code{TRUE}.
+#' }
+#' 
+#' @seealso  \code{\link{nimSvd}} for singular value decompositions in NIMBLE. 
+#' 
+#' @examples
+#' eigenvaluesDemoFunction <- nimbleFunction(
+#'    setup = function(){
+#'      demoMatrix <- diag(4) + 2
+#'    },
+#'    run = function(){
+#'      eigenvalues <- eigen(demoMatrix)$values
+#'      returnType(double(1))
+#'      return(eigenvalues)
+#'  })
+#' 
+nimEigen <- function(x, only.values = FALSE) {
+  ## placeholder list with correct names of elements, will be populated in C++
+  .Call('C_nimEigen', x, as.logical(only.values), eigenNimbleList$new())
+}
+
+
+
+#' Singular Value Decomposition of a Matrix  
+#'
+#' Computes singular values and, optionally, left and right singular vectors of a numeric matrix.
+#' 
+#' @param x a symmetric numeric matrix (double or integer) whose spectral decomposition is to be computed.
+#' @param vectors character that determines whether to calculate left and right singular vectors.  Can take values \code{'none'}, \code{'thin'} or \code{'full'}.  Defaults to \code{'full'}.  See details.
+#'
+#' @author NIMBLE development team
+#'
+#' @aliases svd
+#'
+#' @export
+#'
+#' @details
+#' Computes the singular value decomposition of a numeric matrix using the Eigen C++ template library.  
+#' 
+#' The \code{vectors} character argument determines whether to compute no left and right singular vectors (\code{'none'}), thinned left and right singular vectors (\code{'thin'}), or full left and right singular vectors (\code{'full'}).  For a
+#' matrix \code{x} with dimensions \code{n} and \code{p}, setting \code{vectors = 'thin'} will does the following (quoted from eigen website): 
+#' In case of a rectangular n-by-p matrix, letting m be the smaller value among n and p, there are only m singular vectors; 
+#' the remaining columns of U and V do not correspond to actual singular vectors. 
+#' Asking for thin U or V means asking for only their m first columns to be formed. 
+#' So U is then a n-by-m matrix, and V is then a p-by-m matrix. 
+#' Notice that thin U and V are all you need for (least squares) solving.
+#' 
+#' Setting \code{vectors = 'full'} will compute full matrices for U and V, so that U will be of size n-by-n, and V will be of size p-by-p.
+#' 
+#' In a \code{nimbleFunction}, \code{svd} is identical to \code{nimSvd}. 
+#'  
+#'  \code{returnType(svdNimbleList())} can be used within a \code{link{nimbleFunction}} to specify that the function will return a \code{\link{nimbleList}} generated by the \code{nimSvd} function.  \code{svdNimbleList()} can also be used to define a nested \code{\link{nimbleList}} element.  See the User Manual for usage examples. 
+#' 
+#' @return
+#'  The singular value decomposition of \code{x} is returned as a \code{\link{nimbleList}} with elements:
+#' \itemize{
+#' \item d length m vector containing the singular values of \code{x}, sorted in decreasing order.
+#' \item v matrix with columns containing the left singular vectors of \code{x}, or an empty matrix if \code{vectors = 'none'}.
+#' \item u matrix with columns containing the right singular vectors of \code{x}, or an empty matrix if \code{vectors = 'none'}.
+#' }
+#' 
+#' @seealso  \code{\link{nimEigen}} for spectral decompositions. 
+#' @examples 
+#'  singularValuesDemoFunction <- nimbleFunction(
+#'    setup = function(){
+#'      demoMatrix <- diag(4) + 2
+#'    },
+#'    run = function(){
+#'      singularValues <- svd(demoMatrix)$d
+#'      returnType(double(1))
+#'      return(singularValues)
+#'  })
+nimSvd <- function(x, vectors = 'full') {
+  ## placeholder list with correct names of elements, will be populated in C++
+  vectors <- switch(tolower(vectors),
+                    none = 0,
+                    thin = 1,
+                    full = 2)
+  if(is.null(vectors)) stop("nimSvd: vectors argument to 'svd' must be one of \"none\", \"thin\", or \"full\".")
+  .Call('C_nimSvd', x, vectors, svdNimbleList$new())
+}
+
+
+
+
+
 #' The Dirichlet Distribution
 #'
 #' Density and random generation for the Dirichlet distribution
@@ -244,7 +362,9 @@ NULL
 dmnorm_chol <- function(x, mean, cholesky, prec_param = TRUE, log = FALSE) {
   # cholesky should be upper triangular
   # FIXME: allow cholesky to be lower tri
-  .Call('C_dmnorm_chol', as.double(x), as.double(mean), as.double(cholesky), as.double(prec_param), as.logical(log))
+    if(storage.mode(cholesky) != 'double')
+         storage.mode(cholesky) <- 'double'
+    .Call('C_dmnorm_chol', as.double(x), as.double(mean), cholesky, as.double(prec_param), as.logical(log))
 }
 
 #' @rdname MultivariateNormal
@@ -253,7 +373,9 @@ rmnorm_chol <- function(n = 1, mean, cholesky, prec_param = TRUE) {
  ## cholesky should be upper triangular
  ## FIXME: allow cholesky to be lower tri
     if(n != 1) warning('rmnorm_chol only handles n = 1 at the moment')
-    .Call('C_rmnorm_chol', as.double(mean), as.double(cholesky), as.double(prec_param))
+    if(storage.mode(cholesky) != 'double')
+         storage.mode(cholesky) <- 'double'
+    .Call('C_rmnorm_chol', as.double(mean), cholesky, as.double(prec_param))
 }
 
 #' The Multivariate t Distribution
@@ -292,7 +414,9 @@ NULL
 dmvt_chol <- function(x, mu, cholesky, df, prec_param = TRUE, log = FALSE) {
   # cholesky should be upper triangular
   # FIXME: allow cholesky to be lower tri
-  .Call('C_dmvt_chol', as.double(x), as.double(mu), as.double(cholesky),
+    if(storage.mode(cholesky) != 'double')
+       storage.mode(cholesky) <- 'double'
+    .Call('C_dmvt_chol', as.double(x), as.double(mu), cholesky,
         as.double(df), as.double(prec_param), as.logical(log))
 }
 
@@ -301,8 +425,10 @@ dmvt_chol <- function(x, mu, cholesky, df, prec_param = TRUE, log = FALSE) {
 rmvt_chol <- function(n = 1, mu, cholesky, df, prec_param = TRUE) {
   ## cholesky should be upper triangular
   ## FIXME: allow cholesky to be lower tri
-  if(n != 1) warning('rmnorm_chol only handles n = 1 at the moment')
-  .Call('C_rmvt_chol', as.double(mu), as.double(cholesky),
+    if(n != 1) warning('rmvt_chol only handles n = 1 at the moment')
+    if(storage.mode(cholesky) != 'double')
+         storage.mode(cholesky) <- 'double'
+    .Call('C_rmvt_chol', as.double(mu), cholesky,
         as.double(df), as.double(prec_param))
 }
 
@@ -477,3 +603,96 @@ qexp_nimble <- function(p, rate = 1/scale, scale = 1, lower.tail = TRUE, log.p =
   .Call('C_qexp_nimble', as.double(p), as.double(rate), as.logical(lower.tail), as.logical(log.p))
 }
 
+#' The Inverse Gamma Distribution
+#'
+#'   Density, distribution function, quantile function and random
+#'   generation for the inverse gamma distribution with rate
+#'     or scale (mean = scale / (shape - 1)) parameterizations.
+#' 
+#' @name Inverse-Gamma
+#' 
+#' @param x vector of values.
+#' @param n number of observations.
+#' @param p vector of probabilities.
+#' @param q vector of quantiles.
+#' @param shape vector of shape values, must be positive.
+#' @param rate vector of rate values, must be positive.
+#' @param scale vector of scale values, must be positive.
+#' @param log logical; if TRUE, probability density is returned on the log scale.
+#' @param log.p logical; if TRUE, probabilities p are given by user as log(p).
+#' @param lower.tail logical; if TRUE (default) probabilities are \eqn{P[X \le x]}; otherwise, \eqn{P[X > x]}.
+#' @author Christopher Paciorek
+#' @export
+#' @details The inverse gamma distribution with parameters \code{shape} \eqn{=\alpha}{= a} and
+#' \code{scale} \eqn{=\sigma}{= s} has density
+#' \deqn{
+#'   f(x)= \frac{s^a}{\Gamma(\alpha)} {x}^{-(\alpha+1)} e^{-\sigma/x}%
+#'  }{f(x)= (s^a / Gamma(a)) x^-(a+1) e^-(s/x)}
+#' for \eqn{x \ge 0}, \eqn{\alpha > 0}{a > 0} and \eqn{\sigma > 0}{s > 0}.
+#' (Here \eqn{\Gamma(\alpha)}{Gamma(a)} is the function implemented by \R's
+#'  \code{\link{gamma}()} and defined in its help.
+#'
+#'  The mean and variance are
+#'  \eqn{E(X) = \frac{\sigma}{\alpha}-1}{E(X) = s/(a-1)} and
+#' \eqn{Var(X) = \frac{\sigma^2}{(\alpha-1)^2 (\alpha-2)}}{Var(X) = s^2 / ((a-1)^2 * (a-2))},
+#' with the mean defined only
+#' for \eqn{\alpha > 1}{a > 1} and the variance only for \eqn{\alpha > 2}{a > 2}.
+#'
+#' See Gelman et al., Appendix A or
+#' the BUGS manual for mathematical details. 
+#' @return \code{dinvgamma} gives the density, \code{pinvgamma} gives the distribution
+#' function, \code{qinvgamma} gives the quantile function, and \code{rinvgamma}
+#' generates random deviates.
+#' @references Gelman, A., Carlin, J.B., Stern, H.S., and Rubin, D.B. (2004) \emph{Bayesian Data Analysis}, 2nd ed. Chapman and Hall/CRC.
+#' @seealso \link{Distributions} for other standard distributions
+#' 
+#' @examples
+#' x <- rinvgamma(50, shape = 1, scale = 3)
+#' dinvgamma(x, shape = 1, scale = 3)
+NULL
+
+
+#' @rdname Inverse-Gamma
+#' @export
+dinvgamma <- function(x, shape, scale = 1, rate = 1/scale, log = FALSE) {
+    if (!missing(rate) && !missing(scale)) {
+        if (abs(rate * scale - 1) < 1e-15) 
+            warning("specify 'rate' or 'scale' but not both")
+        else stop("specify 'rate' or 'scale' but not both")
+    }
+    .Call('C_dinvgamma', as.double(x), as.double(shape), as.double(rate), as.logical(log))
+}
+
+#' @rdname Inverse-Gamma
+#' @export
+rinvgamma <- function(n = 1, shape, scale = 1, rate = 1/scale) {
+    if (!missing(rate) && !missing(scale)) {
+        if (abs(rate * scale - 1) < 1e-15) 
+            warning("specify 'rate' or 'scale' but not both")
+        else stop("specify 'rate' or 'scale' but not both")
+    }
+    .Call('C_rinvgamma', as.integer(n), as.double(shape), as.double(rate))
+}
+
+#' @rdname Inverse-Gamma
+#' @export
+pinvgamma <- function(q, shape, scale = 1, rate = 1/scale, lower.tail = TRUE, log.p = FALSE) {
+    if (!missing(rate) && !missing(scale)) {
+        if (abs(rate * scale - 1) < 1e-15) 
+            warning("specify 'rate' or 'scale' but not both")
+        else stop("specify 'rate' or 'scale' but not both")
+    }
+  .Call('C_pinvgamma', as.double(q), as.double(shape), as.double(rate), as.logical(lower.tail), as.logical(log.p))
+}
+
+#' @rdname Inverse-Gamma
+#' @export
+qinvgamma <- function(p, shape, scale = 1, rate = 1/scale, lower.tail = TRUE, log.p = FALSE) {
+    if (!missing(rate) && !missing(scale)) {
+        if (abs(rate * scale - 1) < 1e-15) 
+            warning("specify 'rate' or 'scale' but not both")
+        else stop("specify 'rate' or 'scale' but not both")
+    }
+  .Call('C_qinvgamma', as.double(p), as.double(shape), as.double(rate), as.logical(lower.tail), as.logical(log.p))
+}
+
