@@ -67,7 +67,8 @@ distributionsInputList <- list(
     
     dbeta   = list(BUGSdist = 'dbeta(shape1, shape2, mean, sd)',
                    Rdist    = 'dbeta(shape1 = mean^2*(1-mean)/sd^2-mean, shape2 = mean*(1-mean)^2/sd^2+mean-1)',
-                   altParams= c('mean = shape1/(shape1+shape2)', 'sd = sqrt(shape1*shape2/((shape1 + shape2)^2*(shape1+shape2+1)))'),
+                   altParams= c('mean = shape1/(shape1+shape2)',
+                                'sd = sqrt(shape1*shape2/((shape1 + shape2)^2*(shape1+shape2+1)))'),
                    range    = c(0, 1),
                    pqAvail  = TRUE),
     
@@ -86,8 +87,22 @@ distributionsInputList <- list(
                    pqAvail = TRUE),
     
     dgamma  = list(BUGSdist = 'dgamma(shape, rate, scale, mean, sd)',
-                   Rdist    = c('dgamma(shape, scale = 1/rate)', 'dgamma(shape = mean^2/sd^2, scale = sd^2/mean)'),
-                   altParams= c('rate = 1/scale', 'mean = scale*shape', 'sd = scale * sqrt(shape)'),
+                   Rdist    = c('dgamma(shape, scale = 1/rate)',
+                                'dgamma(shape = mean^2/sd^2, scale = sd^2/mean)'),
+                   altParams= c('rate = 1/scale',
+                                'mean = scale*shape',
+                                'sd = scale * sqrt(shape)'),
+                   range    = c(0, Inf),
+                   pqAvail = TRUE),
+
+    # (shape,scale) is BUGSdist as scale provides conjugacy
+    # calculation of shape/scale from mean/sd not obvious
+    # (solution to cubic polynomial) so not using as alternative param
+    dinvgamma  = list(BUGSdist = 'dinvgamma(shape, scale, rate)',
+                   Rdist    = c('dinvgamma(shape, rate = 1/scale)'),
+                   altParams= c('scale = 1/rate',
+                                'mean = 1 / (rate * (max(shape,1)-1))',
+                                'sd = 1 / (rate * (max(shape,1)-1) * sqrt(max(shape,2)-2))'), # max ensures Inf moment when appropriate
                    range    = c(0, Inf),
                    pqAvail = TRUE),
     
@@ -95,8 +110,9 @@ distributionsInputList <- list(
     
     dlnorm  = list(BUGSdist = 'dlnorm(meanlog, taulog, sdlog, varlog)',
                    Rdist    = c('dlnorm(meanlog, sdlog = 1/sqrt(taulog))',
-                       'dlnorm(meanlog, sdlog = sqrt(varlog))'),
-                   altParams= c('taulog = sdlog^-2', 'varlog = sdlog^2'),
+                                'dlnorm(meanlog, sdlog = sqrt(varlog))'),
+                   altParams= c('taulog = sdlog^-2',
+                                'varlog = sdlog^2'),
                    range    = c(0, Inf),
                    pqAvail = TRUE),
     
@@ -106,25 +122,31 @@ distributionsInputList <- list(
                    pqAvail = TRUE),
     
     dnorm   = list(BUGSdist = 'dnorm(mean, tau, sd, var)',
-                   Rdist    = c('dnorm(mean, sd = 1/sqrt(tau))', 'dnorm(mean, sd = sqrt(var))'),
-                   altParams= c('tau = sd^-2', 'var = sd*sd'),
+                   Rdist    = c('dnorm(mean, sd = 1/sqrt(tau))',
+                                'dnorm(mean, sd = sqrt(var))'),
+                   altParams= c('tau = sd^-2',
+                                'var = sd*sd'),
                    pqAvail = TRUE),
     
     dt      = list(BUGSdist = 'dt(mu, tau, df, sigma, sigma2)',
                    Rdist    = c('dt_nonstandard(df, mu, sigma = 1/sqrt(tau))',
-                       'dt_nonstandard(df, mu, sigma = sqrt(sigma2))'),
-                   altParams = c('tau = sigma^-2', 'sigma2 = sigma^2'),
+                                'dt_nonstandard(df, mu, sigma = sqrt(sigma2))'),
+                   altParams = c('tau = sigma^-2',
+                                 'sigma2 = sigma^2'),
                    pqAvail = TRUE),
     
     dunif   = list(BUGSdist = 'dunif(min, max, mean, sd)',
                    Rdist    = c('dunif(min = mean - sqrt(3)*sd, max = mean + sqrt(3)*sd)'),
-                   altParams = c('mean = (min + max)/2', 'sd = (max - min)/sqrt(12)'),
+                   altParams = c('mean = (min + max)/2',
+                                 'sd = (max - min)/sqrt(12)'),
                    range = c('lower = min', 'upper = max'),
                    pqAvail = TRUE),
     
     dweib   = list(BUGSdist = 'dweib(shape, lambda, scale, rate)',
-                   Rdist    = c('dweibull(shape, scale = lambda^(-1/shape))', 'dweibull(shape, scale = 1/rate)'),
-                   altParams= c('rate = 1/scale', 'lambda = scale^(-shape)'),
+                   Rdist    = c('dweibull(shape, scale = lambda^(-1/shape))',
+                                'dweibull(shape, scale = 1/rate)'),
+                   altParams= c('rate = 1/scale',
+                                'lambda = scale^(-shape)'),
                    range   = c(0, Inf),
                    pqAvail = TRUE,
                    alias = 'dweibull'),
@@ -142,8 +164,11 @@ distributionsInputList <- list(
                    alias    = 'ddirich'),
     
     dmnorm  = list(BUGSdist = 'dmnorm(mean, prec, cov, cholesky, prec_param)',
-                   Rdist    = c('dmnorm_chol(mean, cholesky = chol(prec), prec_param = 1)', 'dmnorm_chol(mean, cholesky = chol(cov), prec_param = 0)', 'dmnorm_chol(mean, cholesky, prec_param)'),
-                   altParams= c('prec = calc_dmnormAltParams(cholesky, prec_param, 1)', 'cov = calc_dmnormAltParams(cholesky, prec_param, 0)'),
+                   Rdist    = c('dmnorm_chol(mean, cholesky = chol(prec), prec_param = 1)',
+                                'dmnorm_chol(mean, cholesky = chol(cov), prec_param = 0)',
+                                'dmnorm_chol(mean, cholesky, prec_param)'),
+                   altParams= c('prec = calc_dmnormAltParams(cholesky, prec_param, 1)',
+                                'cov = calc_dmnormAltParams(cholesky, prec_param, 0)'),
                    types    = c('value = double(1)', 'mean = double(1)', 'cholesky = double(2)', 'prec = double(2)', 'cov = double(2)')),
     
     dmulti  = list(BUGSdist = 'dmulti(prob, size)',
@@ -154,14 +179,18 @@ distributionsInputList <- list(
                    alias    = 'dmultinom'),
     
     dmvt  = list(BUGSdist = 'dmvt(mu, prec, scale, cholesky, df, prec_param)',
-                   Rdist    = c('dmvt_chol(mu, cholesky = chol(prec), df = df, prec_param = 1)', 'dmvt_chol(mu, cholesky = chol(scale), df = df, prec_param = 0)', 'dmvt_chol(mu, cholesky, df = df, prec_param)'),
-                   ## I think it's okay to use the same altParams function for multivariate normal and t
-                   altParams= c('prec = calc_dmnormAltParams(cholesky, prec_param, 1)', 'scale = calc_dmnormAltParams(cholesky, prec_param, 0)'),
+                   Rdist    = c('dmvt_chol(mu, cholesky = chol(prec), df = df, prec_param = 1)',
+                                'dmvt_chol(mu, cholesky = chol(scale), df = df, prec_param = 0)',
+                                'dmvt_chol(mu, cholesky, df = df, prec_param)'),
+                   altParams= c('prec = calc_dmnormAltParams(cholesky, prec_param, 1)',
+                                'scale = calc_dmnormAltParams(cholesky, prec_param, 0)'),
                    types    = c('value = double(1)', 'mu = double(1)', 'cholesky = double(2)', 'df = double(0)', 'prec = double(2)', 'scale = double(2)')),
     
     dwish   = list(BUGSdist = 'dwish(R, df, S)',
-                   Rdist    = c('dwish_chol(cholesky = chol(R), df, scale_param = 0)', 'dwish_chol(cholesky = chol(S), df, scale_param = 1)'),
-                   altParams = c('R = inverse(t(cholesky)%*%cholesky)', 'S = inverse(t(cholesky)%*%cholesky)'),
+                   Rdist    = c('dwish_chol(cholesky = chol(R), df, scale_param = 0)',
+                                'dwish_chol(cholesky = chol(S), df, scale_param = 1)'),
+                   altParams = c('R = inverse(t(cholesky)%*%cholesky)',
+                                 'S = inverse(t(cholesky)%*%cholesky)'),
                    alias    = 'dwishart',
                    types    = c('value = double(2)', 'R = double(2)', 'S = double(2)', 'cholesky = double(2)'))
     # altParams ok here (but still would like back/forwardsolve), because code in altParams only used for R if S was provided by user and vice versa, so don't need 'if'
