@@ -21,6 +21,7 @@
 ## 2. nimbleFunctions that return non-scalar-equivalent and are used within a larger expression
 
 exprClasses_labelForEigenization <- function(code) {
+
     if(code$isCall) {
         if(code$name == '{') {
             for(i in seq_along(code$args)) {
@@ -66,55 +67,76 @@ exprClasses_labelForEigenization <- function(code) {
 
 ## EIGEN DOES NOT DO AUTOMATIC TYPE PROMOTION
 
-callsFromExternalUnaries <- as.character(unlist(lapply(eigProxyTranslateExternalUnary, `[`, 1)))
+callsFromExternalUnaries <- as.character(unlist(lapply(eigProxyTranslateExternalUnary, function(x) if(length(x) < 4) x[1] else x[4])))
 
 eigenizeCalls <- c( ## component-wise unarys valid for either Eigen array or matrix
-    makeCallList(c('abs','square','sqrt','('), 'eigenize_cWiseUnaryEither'),
+    makeCallList(c('abs','square','sqrt','(','t'), 'eigenize_cWiseUnaryEither'),
     makeCallList('pow', 'eigenize_cWiseByScalarArray'),
-                   makeCallList(c('asRow', 'asCol'), 'eigenize_asRowOrCol'),
-                      ## component-wise unarys valid only for only Eigen array
-                   makeCallList(c('exp','log','cube','cwiseInverse','sin','cos','tan','asin','acos', callsFromExternalUnaries), 'eigenize_cWiseUnaryArray'), 
-                   
-                   ## component-wise binarys valid for either Eigen array or matrix, but the arguments must match
-                   ## Check on which if any of these can extend to a scalar on one side
-                   makeCallList(c('pmin','pmax'), 'eigenize_cWiseBinaryArray'),
-
-                   ## component-wise multiplication or division
-                   makeCallList(c('*','/'), 'eigenize_cWiseMultDiv'),
-                   
-                   ## component-wise addition or subtraction
-                   makeCallList(c('+','-'), 'eigenize_cWiseAddSub'),
-
-                   makeCallList(reductionUnaryOperatorsEither, 'eigenize_reductionEither'), ##'eigenize_cWiseUnaryEither'),
-                   makeCallList(reductionUnaryOperatorsArray, 'eigenize_reductionArray'),
-                   makeCallList(reductionBinaryOperatorsEither, 'eigenize_reductionBinaryEither'),
-                   makeCallList(c('%*%'), 'eigenize_cWiseBinaryMatrix'),
-                      ## matrix ops
+    makeCallList(c('asRow', 'asCol'), 'eigenize_asRowOrCol'),
+    ## component-wise unarys valid only for only Eigen array
+    makeCallList(c('exp','log','cube','cwiseInverse','sin','cos','tan','asin','acos', '!', callsFromExternalUnaries), 'eigenize_cWiseUnaryArray'), 
+    
+    ## component-wise binarys valid for either Eigen array or matrix, but the arguments must match
+    ## Check on which if any of these can extend to a scalar on one side
+    makeCallList(c('pmin','pmax'), 'eigenize_cWiseBinaryArray'),
+    makeCallList(binaryMidLogicalOperators, 'eigenize_cWiseBinaryArrayLogical'),
+    
+    ## component-wise multiplication or division
+    makeCallList(c('*','/'), 'eigenize_cWiseMultDiv'),
+    
+    ## component-wise addition or subtraction
+    makeCallList(c('+','-'), 'eigenize_cWiseAddSub'),
+    
+    makeCallList(reductionUnaryOperatorsEither, 'eigenize_reductionEither'), 
+    makeCallList(reductionUnaryOperatorsArray, 'eigenize_reductionArray'),
+    makeCallList(reductionBinaryOperatorsEither, 'eigenize_reductionBinaryEither'),
+    makeCallList(c('%*%'), 'eigenize_cWiseBinaryMatrix'),
+    ## matrix ops
     makeCallList(matrixSolveOperators, 'eigenize_matrixOps'),
-    list('t' = 'eigenize_cWiseUnaryEither',
+    makeCallList(scalar_distribution_dFuns, 'eigenize_recyclingRuleFunction'),
+    makeCallList(scalar_distribution_pFuns, 'eigenize_recyclingRuleFunction'),
+    makeCallList(scalar_distribution_qFuns, 'eigenize_recyclingRuleFunction'),
+    makeCallList(scalar_distribution_rFuns, 'eigenize_recyclingRuleFunction'),
+    makeCallList(c(paste0(c('d','r','q','p'), 't'), paste0(c('d','r','q','p'), 'exp')) , 'eigenize_recyclingRuleFunction'),
+    makeCallList(coreRnonSeqBlockCalls, 'eigenize_nonSeq'),
+    makeCallList(coreRmanipulationCalls, 'eigenize_nimbleNullaryClass'),
+    makeCallList(c('nimCd','nimCi','nimCb'), 'eigenize_alwaysMatrix'),
+    list(##'t' = 'eigenize_cWiseUnaryEither',
+        ':' = 'eigenize_nimbleNullaryClass', ## at this point ':' is like a coreRmanipulationCall
+         eigenBlock = 'eigenize_eigenBlock',
+         diagonal  = 'eigenize_cWiseUnaryMatrix',
          'inverse' = 'eigenize_cWiseUnaryMatrix',
-         'chol' = 'eigenize_matrixOps'
+         'chol' = 'eigenize_matrixOps',
+         RRtest_add = 'eigenize_recyclingRuleFunction'
          )
 )
 
-eigenizeCallsBeforeRecursing <- c( ## These cannot be calls that trigger aliasRisk. ## getParam/getBound always triggers an intermediate so it should never need handling here
-    makeCallList(c('size','nimArr_dmnorm_chol', 'nimArr_dmvt_chol', 'nimArr_dwish_chol', 'nimArr_ddirch','calculate','calculateDiff','getLogProb', 'getParam', 'getBound','getNodeFunctionIndexedInfo'), 'eigenize_doNotRecurse'),
-    list(nfVar = 'eigenize_nfVar',
+eigenizeCallsBeforeRecursing <- c( ## These cannot be calls that trigger aliasRisk. ## getParam always triggers an intermediate so it should never need handling here
+    makeCallList(c('size','nimArr_dmnorm_chol', 'nimArr_dmvt_chol', 'nimArr_dwish_chol', 'nimArr_ddirch','calculate','calculateDiff','getLogProb', 'getParam', 'getBound', 'getNodeFunctionIndexedInfo', 'concatenateTemp', 'MAKE_FIXED_VECTOR', 'hardCodedVectorInitializer'), 'eigenize_doNotRecurse'),
+    list(coeffSetter = 'eigenize_coeffSetter',
+         nfVar = 'eigenize_nfVar',
          chainedCall = 'eigenize_chainedCall',
          '<-' = 'eigenize_assign_before_recurse',
-         mvAccessRow = 'eigenize_nfVar') )
+         mvAccessRow = 'eigenize_nfVar') )##,
+         ##setWhich = 'eigenize_setWhich') )
+
+eigenizeUseArgs <- c(
+    list(
+        setWhich = c(FALSE, TRUE),
+        setRepVectorTimes = c(FALSE, TRUE, TRUE)
+        ))
 
 ## This is a list of translations for the C++ code generation system.
 ## e.g. if abs(X) gets eigenized, it is turned into cwiseAbs(X)
 ## which in nimGenerateCpp is generated as X.cwiseAbs()
 eigenizeTranslate <- list(abs = 'cwiseAbs',
-                       square = 'cwiseAbs2',
-                       sqrt = 'cwiseSqrt',
+                          square = 'cwiseAbs2',
+                          sqrt = 'cwiseSqrt',
                           inprod = 'eiginprod',
                           
-                       asRow = 'eigBlank',
-                       asCol = 'eigBlank',
-                       
+                          asRow = 'eigBlank',
+                          asCol = 'eigBlank',
+                          
                           exp = 'eigExp',
                           log = 'eigLog',
                           pow = 'eigPow',
@@ -126,24 +148,26 @@ eigenizeTranslate <- list(abs = 'cwiseAbs',
                           asin = 'eigAsin',
                           acos = 'eigAcos',
                           inverse = 'eigInverse',
-
-                       pmin = 'eigpmin',
-                       pmax = 'eigpmax',
-
-                       '*' = 'cwiseProduct', ## detailed handling in eigenize_cWiseMultDiv
-                       '/' = 'cwiseQuotient',
-                       
-                       '+' = '+',
-                       '-' = '-',
-
-                       ## matrix operations
-                       '%*%' = '*',
-                       't' = 'eigTranspose',
+                          
+                          pmin = 'eigpmin',
+                          pmax = 'eigpmax',
+                          diagonal = 'eigDiagonal',
+                          
+                          '*' = 'cwiseProduct', ## detailed handling in eigenize_cWiseMultDiv
+                          '/' = 'cwiseQuotient',
+                          
+                          '+' = '+',
+                          '-' = '-',
+                          
+                          ## matrix operations
+                          '%*%' = '*',
+                          't' = 'eigTranspose',
                           'det' = 'eigDeterminant',
-                       
-                       '<-' = '<-',
-                       '(' = '('
-                       )
+                          
+                          '<-' = '<-',
+                          '(' = '(',
+                          eigenBlock = 'eigenBlock' ## always inserted by sizeIndexingBracket as 'eigenBlock'
+                          )
 
 ETentriesFromExternalUnaries <- as.list(names(eigProxyTranslateExternalUnary))
 names(ETentriesFromExternalUnaries) <- callsFromExternalUnaries
@@ -151,6 +175,7 @@ eigenizeTranslate <- c(eigenizeTranslate, ETentriesFromExternalUnaries)
 
 for(rop in reductionUnaryOperators) eigenizeTranslate[[rop]] <- paste0('eig', rop)
 for(rop in matrixSquareReductionOperators) eigenizeTranslate[[rop]] <- paste0('eig', rop)
+for(rop in binaryMidLogicalOperators) eigenizeTranslate[[rop]] <- rop
 
 ## scalar*matrix, matrix*scalar and matrix/scalar allowed
 ## ditter with matrix *= scalar and matrix /= scalar
@@ -187,7 +212,8 @@ exprClasses_eigenize <- function(code, symTab, typeEnv, workEnv = new.env()) {
     setupExprs <- list()
     if(code$isName) {
         ## Generate EigenMap and "new" assignment
-        setupExprs <- c(setupExprs, eigenizeName(code, symTab, typeEnv, workEnv))
+##        if(code$name != "") ## can happen in a blank index, e.g. x[,i] ## actually we'll fill in blanks in sizeIndexingBracket
+            setupExprs <- c(setupExprs, eigenizeName(code, symTab, typeEnv, workEnv))
     }
     if(code$isCall) {
         if(code$name == '{') {
@@ -225,7 +251,7 @@ exprClasses_eigenize <- function(code, symTab, typeEnv, workEnv = new.env()) {
             setupExprs <- c(setupExprs, eigenizeNameStrided(code, symTab, typeEnv, workEnv))
             return(setupExprs)
         }
-        if(code$name == '[') { ## If there is still A[i] in the code, it is because it ces to a scalar and does not need eigenization
+        if(code$name == '[') { ## If there is still A[i] in the code, it is because it is equivalent to a scalar and does not need eigenization
             if(code$nDim == 0) {
 ##                writeLines(paste0('Not eigenizing ', nimDeparse(code)))
                 return(NULL)
@@ -242,12 +268,18 @@ exprClasses_eigenize <- function(code, symTab, typeEnv, workEnv = new.env()) {
 
         IsetAliasRisk <- FALSE
         if(code$name %in% c('t', 'asRow')) {IsetAliasRisk <- workEnv[['aliasRisk']] <- TRUE}
+
+
+        iArgs <- seq_along(code$args)
+        useArgs <- eigenizeUseArgs[[code$name]]
+        if(!is.null(useArgs)) iArgs <- iArgs[-which(!useArgs)] ## this allows iArgs to be longer than useArgs.  if equal length, iArgs[useArgs] would work 
         
-        for(i in seq_along(code$args)) {
+        for(i in iArgs) {
             if(inherits(code$args[[i]], 'exprClass'))
                 setupExprs <- c(setupExprs, exprClasses_eigenize(code$args[[i]], symTab, typeEnv, workEnv))
         }
         ## finally, call any special handlers
+        
         eCall <- eigenizeCalls[[code$name]]
                 
         if(!is.null(eCall)) {
@@ -263,6 +295,10 @@ eigenize_doNotRecurse <- function(code, symTab, typeEnv, workEnv) {
     invisible(NULL)
 }
 
+## eigenize_setWhich <- function(code, symTab, typeEnv, workEnv) {## first arg should be left as NimArr
+##     exprClasses_eigenize(code$args[[2]], symTab, typeEnv, workEnv)
+## }
+
 ## eigenizeHandlers
 eigenize_asRowOrCol <- function(code, symTab, typeEnv, workEnv) {
     code$eigMatrix <- code$args[[1]]$eigMatrix
@@ -284,10 +320,14 @@ eigenize_reductionBinaryEither <- function(code, symTab, typeEnv, workEnv) {
     newName <- eigenizeTranslate[[code$name]]
     if(is.null(newName)) stop(exprClassProcessingErrorMsg(code, 'Missing eigenizeTranslate entry.'), call. = FALSE)
     code$name <- newName
-##    makeEigenArgsMatch(code)
-##    for(i in 1:2) 
-##        if(code$args[[i]]$eigMatrix) eigenizeArrayize(code$args[[i]])
+    promoteTypes(code)
+    invisible(NULL)
+}
 
+eigenize_recyclingRuleFunction <- function(code, symTab, typeEnv, workEnv) {
+    if(code$nDim == 0) return(NULL)
+    code$eigMatrix <- TRUE
+    code$name <- paste0(code$name, '_RR_impl<MatrixXd>::', code$name,'_RecyclingRule')
     invisible(NULL)
 }
 
@@ -295,7 +335,7 @@ eigenize_reductionEither <- function(code, symTab, typeEnv, workEnv) {
     newName <- eigenizeTranslate[[code$name]]
     if(is.null(newName)) stop(exprClassProcessingErrorMsg(code, 'Missing eigenizeTranslate entry.'), call. = FALSE)
     code$name <- newName
-##    code$eigMatrix <- code$args[[1]]$eigMatrix
+    promoteTypes(code)
     invisible(NULL)
 }
 
@@ -303,9 +343,63 @@ eigenize_reductionArray <- function(code, symTab, typeEnv, workEnv) {
     newName <- eigenizeTranslate[[code$name]]
     if(is.null(newName)) stop(exprClassProcessingErrorMsg(code, 'Missing eigenizeTranslate entry.'), call. = FALSE)
     code$name <- newName
-##    code$eigMatrix <- code$args[[1]]$eigMatrix
     if(length(code$args[[1]]$eigMatrix) == 0) stop(paste0("Trying it eigenize ", nimDeparse(code), " but information from the argument is not complete."), call. = FALSE)
     if(code$args[[1]]$eigMatrix) eigenizeArrayize(code$args[[1]])
+    promoteTypes(code)
+    invisible(NULL)
+}
+
+eigenize_nimbleNullaryClass <- function(code, symTab, typeEnv, workEnv) {
+    if(code$nDim == 0) return(NULL)
+    code$eigMatrix <- TRUE
+    invisible(NULL)
+}
+
+eigenize_alwaysMatrix <- function(code, symTab, typeenv, workEnv) {
+    code$eigMatrix <- TRUE
+    invisible(NULL)
+}
+
+eigenize_nonSeq <- function(code, symTab, typeEnv, workEnv) {
+    dropBool <- TRUE
+    if(!is.null(names(code$args)))
+        if('drop' %in% names(code$args)) {
+            iDropArg <- which(names(code$args)=='drop')
+            dropBool <- code$args[[iDropArg]]
+            code$args[[iDropArg]] <- NULL
+        }
+    ans <- eigenize_nimbleNullaryClass(code, symTab, typeEnv, workEnv)
+    origCodeCaller <- code$caller
+    origCodeCallerArgID <- code$callerArgID
+    newExpr <- addTransposeIfNeededForNonSeqBlock(code, dropBool)
+    if(!identical(newExpr, code)) {
+        setArg(origCodeCaller, origCodeCallerArgID, newExpr)
+        newExpr$name <- 't' ## this is set to eigTranspose for the other case, but to call the handler it needs to be 't'!
+        ans2 <- eval(call(eigenizeCalls[['t']], newExpr, symTab, typeEnv, workEnv))
+        c(ans2, ans)
+    } else
+        ans
+}
+
+eigenize_eigenBlock <- function(code, symTab, typeEnv, workEnv) {
+    ## re-arrange from i:j to i,j
+    ## we do this here rather than in size processing so that eigenBlock and nimNonSeqX can maintain same argument format during size processing
+    dropBool <- TRUE
+    if(!is.null(names(code$args)))
+        if('drop' %in% names(code$args)) {
+            iDropArg <- which(names(code$args)=='drop')
+            dropBool <- code$args[[iDropArg]]
+            code$args[[iDropArg]] <- NULL
+        }
+            
+    newExpr <- makeEigenBlockExprFromBrackets(code, drop = dropBool) ## at this point it is ok that code exprClass is messed up (first arg re-used in newExpr)
+    newExpr$sizeExprs <- code$sizeExprs
+    newExpr$type <- code$type
+    newExpr$nDim <- code$nDim
+    newExpr$toEigenize <- 'yes'
+    newExpr$eigMatrix <- code$args[[1]]$eigMatrix
+    setArg(code$caller, code$callerArgID, newExpr)
+    ## note that any expressions like sum(A) in 1:sum(A) should have already been lifted
     invisible(NULL)
 }
 
@@ -315,22 +409,87 @@ eigenize_cWiseUnaryEither <- function(code, symTab, typeEnv, workEnv) {
     if(is.null(newName)) stop(exprClassProcessingErrorMsg(code, 'Missing eigenizeTranslate entry.'), call. = FALSE)
     code$name <- newName
     code$eigMatrix <- code$args[[1]]$eigMatrix
+    promoteTypes(code)
     invisible(NULL)
 }
 
+eigenize_coeffSetter <- function(code, symTab, typeEnv, workEnv) {
+    ## a peculiar case: we expect it to be on LHS but must take control of workEnv$OnLHSnow, which would have been set by eigenize_assign_before_recursing
+    dropBool <- TRUE
+    if(!is.null(names(code$args)))
+        if('drop' %in% names(code$args)) {
+            iDropArg <- which(names(code$args)=='drop')
+            code$args[[iDropArg]] <- NULL
+        }
+    setupExprs <- list()
+    workEnv$OnLHSnow <- TRUE
+    setupExprs <- c(setupExprs, exprClasses_eigenize(code$args[[1]], symTab, typeEnv, workEnv))
+    workEnv$OnLHSnow <- NULL
+    if(inherits(code$args[[2]], 'exprClass')) setupExprs <- c(setupExprs, exprClasses_eigenize(code$args[[2]], symTab, typeEnv, workEnv))
+    if(inherits(code$args[[3]], 'exprClass')) setupExprs <- c(setupExprs, exprClasses_eigenize(code$args[[3]], symTab, typeEnv, workEnv))
+    workEnv$OnLHSnow <- TRUE
+    setupExprs
+}
+
+promoteArgTypes <- function(code) {
+    if(!(inherits(code$args[[1]], 'exprClass') & inherits(code$args[[2]], 'exprClass'))) return(NULL)
+    a1type <- code$args[[1]]$type
+    a2type <- code$args[[2]]$type
+    if(a1type == a2type) return(NULL)
+    if(code$name == '<-') return(eigenCast(code, 2, a1type))
+    if(a2type == 'double') return(eigenCast(code, 1, 'double'))
+    if(a1type == 'double') return(eigenCast(code, 2, 'double'))
+    if(a2type == 'integer') return(eigenCast(code, 1, 'integer'))
+    if(a1type == 'integer') return(eigenCast(code, 2, 'integer'))
+    ## Should never get past here, but for completeness:
+    if(a2type == 'logical') return(eigenCast(code, 1, 'logical'))
+    if(a1type == 'logical') return(eigenCast(code, 2, 'logical'))
+}
+
+promoteTypes <- function(code) {
+    resultType <- code$type
+    for(i in seq_along(code$args)) {
+        if(inherits(code$args[[i]], 'exprClass')) {
+            if(code$args[[i]]$type != resultType) {
+                eigenCast(code, i, resultType)
+            }
+        }
+    }
+    NULL
+}
+
+eigenCast <- function(expr, argIndex, newType) {
+    castExpr <- insertExprClassLayer(expr, argIndex, 'eigenCast',
+                                     sizeExprs = expr$args[[argIndex]]$sizeExprs,
+                                     nDim = expr$args[[argIndex]]$nDim,
+                                     eigMatrix = expr$args[[argIndex]]$eigMatrix)
+    castExpr$args[[2]] <- switch(newType, double = 'double', integer = 'int', logical = 'bool')
+    castExpr$type <- newType
+
+    castExpr
+}
+
 eigenize_assign_before_recurse <- function(code, symTab, typeEnv, workEnv) {
     setupExprs <- list()
     if(length(code$args) != 2) stop(exprClassProcessingErrorMsg(code, 'There is an assignment without 2 arguments.'), call. = FALSE)
     workEnv$OnLHSnow <- TRUE
     setupExprs <- c(setupExprs, exprClasses_eigenize(code$args[[1]], symTab, typeEnv, workEnv))
     workEnv$OnLHSnow <- NULL ## allows workEnv[['OnLHSnow']] to be NULL if is does not exist or if set to NULL
-    setupExprs <- c(setupExprs, exprClasses_eigenize(code$args[[2]], symTab, typeEnv, workEnv))
-
+    changeToFill <- FALSE
+    if(inherits(code$args[[2]], 'exprClass')) {
+        setupExprs <- c(setupExprs, exprClasses_eigenize(code$args[[2]], symTab, typeEnv, workEnv))
+        if(code$args[[2]]$nDim == 0) if(code$args[[1]]$nDim > 0) changeToFill <- TRUE
+    } else {
+        changeToFill <- TRUE
+    }
     if(!is.null(workEnv[['mustAddEigenEval']])) insertExprClassLayer(code, 2, 'eigEval')
-    
-    newName <- eigenizeTranslate[[code$name]]
-    if(is.null(newName)) stop(exprClassProcessingErrorMsg(code, 'Missing eigenizeTranslate entry.'), call. = FALSE)
-    code$name <- newName
+
+    if(changeToFill) code$name <- 'fill'
+    else {
+        newName <- eigenizeTranslate[[code$name]]
+        if(is.null(newName)) stop(exprClassProcessingErrorMsg(code, 'Missing eigenizeTranslate entry.'), call. = FALSE)
+        code$name <- newName
+    }
     code$eigMatrix <- code$args[[1]]$eigMatrix
     setupExprs
 }
@@ -338,7 +497,7 @@ eigenize_assign_before_recurse <- function(code, symTab, typeEnv, workEnv) {
 eigenize_matrixOps <- function(code, symTab, typeEnv, workEnv) {
     if(!code$args[[1]]$eigMatrix) eigenizeMatricize(code$args[[1]])
     if(length(code$args) == 2)
-        if(!code$args[[2]]$eigMatrix) eigenizeMatricize(code$args[[2]])
+      if(!code$args[[2]]$eigMatrix) eigenizeMatricize(code$args[[2]])	
     code$eigMatrix <- TRUE
     code$name <- switch(code$name,
                         chol = 'EIGEN_CHOL',
@@ -347,6 +506,7 @@ eigenize_matrixOps <- function(code, symTab, typeEnv, workEnv) {
                         backsolve = 'EIGEN_BS',
                         stop('should never get here')
                         )
+    promoteTypes(code)
     invisible(NULL)
 }
 
@@ -358,6 +518,7 @@ eigenize_cWiseUnaryArray <- function(code, symTab, typeEnv, workEnv) {
     code$eigMatrix <- FALSE
     if(length(code$args[[1]]$eigMatrix) == 0) stop(exprClassProcessingErrorMsg(code, 'Information for eigenizing was not complete,'), call. = FALSE)
     if(code$args[[1]]$eigMatrix) eigenizeArrayize(code$args[[1]])
+    promoteTypes(code)
     invisible(NULL)
 }
 
@@ -368,6 +529,7 @@ eigenize_cWiseUnaryMatrix <- function(code, symTab, typeEnv, workEnv) {
     code$name <- newName
     code$eigMatrix <- TRUE
     if(!code$args[[1]]$eigMatrix) eigenizeMatricize(code$args[[1]])
+    promoteTypes(code)
     invisible(NULL)
 }
 
@@ -388,7 +550,21 @@ eigenize_cWiseBinaryEitherMatch <- function(code, symTab, typeEnv, workEnv) {
     code$name <- newName
     makeEigenArgsMatch(code)
     code$eigMatrix <- code$args[[1]]$eigMatrix
-	invisible(NULL)
+    promoteTypes(code)
+    invisible(NULL)
+}
+
+
+eigenize_cWiseBinaryArrayLogical <- function(code, symTab, typeEnv, workEnv) {
+    if(code$nDim == 0) return(NULL)
+    newName <- eigenizeTranslate[[code$name]]
+    if(is.null(newName)) stop(exprClassProcessingErrorMsg(code, 'Missing eigenizeTranslate entry.'), call. = FALSE)
+    code$name <- newName
+    code$eigMatrix <- TRUE
+    if(inherits(code$args[[1]], 'exprClass')) if(!isEigScalar(code$args[[1]])) if(code$args[[1]]$eigMatrix) eigenizeArrayize(code$args[[1]])
+    if(inherits(code$args[[2]], 'exprClass')) if(!isEigScalar(code$args[[2]])) if(code$args[[2]]$eigMatrix) eigenizeArrayize(code$args[[2]])
+    promoteArgTypes(code) ## key difference for logical case: promote args to match each other, not logical return type
+    invisible(NULL)
 }
 
 
@@ -398,8 +574,9 @@ eigenize_cWiseBinaryArray <- function(code, symTab, typeEnv, workEnv) {
     if(is.null(newName)) stop(exprClassProcessingErrorMsg(code, 'Missing eigenizeTranslate entry.'), call. = FALSE)
     code$name <- newName
     code$eigMatrix <- TRUE
-    if(code$args[[1]]$eigMatrix) eigenizeArrayize(code$args[[1]])
-    if(code$args[[2]]$eigMatrix) eigenizeArrayize(code$args[[2]])
+    if(inherits(code$args[[1]], 'exprClass')) if(code$args[[1]]$eigMatrix) eigenizeArrayize(code$args[[1]])
+    if(inherits(code$args[[2]], 'exprClass')) if(code$args[[2]]$eigMatrix) eigenizeArrayize(code$args[[2]])
+    promoteTypes(code)
     invisible(NULL)
 }
 
@@ -412,6 +589,7 @@ eigenize_cWiseByScalarArray <- function(code, symTab, typeEnv, workEnv) {
     code$name <- newName
     code$eigMatrix <- FALSE
     if(code$args[[1]]$eigMatrix) eigenizeArrayize(code$args[[1]])
+    promoteTypes(code)
     invisible(NULL)
 }
 
@@ -423,6 +601,7 @@ eigenize_cWiseBinaryMatrix <- function(code, symTab, typeEnv, workEnv) {
     code$eigMatrix <- TRUE
     if(!code$args[[1]]$eigMatrix) eigenizeMatricize(code$args[[1]])
     if(!code$args[[2]]$eigMatrix) eigenizeMatricize(code$args[[2]])
+    promoteTypes(code)
     invisible(NULL)
 }
 
@@ -438,6 +617,7 @@ eigenize_cWiseMultDiv <- function(code, symTab, typeEnv, workEnv) {
  
     if(scalar1 | scalar2) {
         if(scalar1 & scalar2) return(invisible(NULL))
+        promoteTypes(code)
         if(code$name == '*' | scalar2) {
             code$eigMatrix <- if(scalar1) code$args[[2]]$eigMatrix else code$args[[1]]$eigMatrix
             return(invisible(NULL))
@@ -457,6 +637,7 @@ eigenize_cWiseAddSub <- function(code, symTab, typeEnv, workEnv) {
     if(length(code$args)==1) return(eigenize_cWiseUnaryEither(code, symTab, typeEnv, workEnv))
     
     if(isEigScalar(code$args[[1]]) | isEigScalar(code$args[[2]])) {
+        promoteTypes(code)
         for(i in 1:2) { 
             if(!isEigScalar(code$args[[i]])) { 
                 if(code$args[[i]]$eigMatrix) eigenizeArrayize(code$args[[i]])
@@ -514,7 +695,7 @@ EigenNewExpr <- function(EigenName, targetName, offsetExpr = NULL, MatrixType, n
 makeEigenName <- function(name) paste0('Eig_',name)
 
 makeEigenTypeLabel <- function(Matrix = TRUE, baseType = 'double') {
-    paste0(if(Matrix) 'MatrixX' else 'ArrayXX', if(baseType == 'double') 'd' else 'i')
+    paste0(if(Matrix) 'MatrixX' else 'ArrayXX', if(baseType == 'double') 'd' else if(baseType =='integer') 'i' else 'b')
 }
 
 eigenize_chainedCall <- function(code, symTab, typeEnv, workEnv) {
@@ -529,7 +710,7 @@ eigenize_nfVar <- function(code, symTab, typeEnv, workEnv) { ## A lot like eigen
     targetTypeSizeExprs <- code$sizeExprs
 
     if(length(targetTypeSizeExprs) > 2) {
-        stop(exprClassProcessingErrorMsg(code, 'Cannot eigenize a map of dimensions > 2.'), call. = FALSE)
+        stop(exprClassProcessingErrorMsg(code, 'Cannot do math with arrays that have more than 2 dimensions.'), call. = FALSE)
     }
     if(length(targetTypeSizeExprs) == 2) {
         nrowExpr <- targetTypeSizeExprs[[1]]
@@ -550,7 +731,10 @@ eigenize_nfVar <- function(code, symTab, typeEnv, workEnv) { ## A lot like eigen
     targetVarProxy <- EigenName ## don't have a more direct targetVar - a little redundant but keeps the code similar to eigenizeName and eigenizeNameStrided
     thisMapAlreadySet <- FALSE
     if(!is.null(workEnv[['OnLHSnow']])) { ## this is the var on the LHS
-        if(!is.null(workEnv[['LHSeigenName']])) stop(exprClassProcessingErrorMsg(code, 'LHSeigenName already exists.'), call. = FALSE)
+        if(!is.null(workEnv[['LHSeigenName']])) {
+            browser()
+            stop(exprClassProcessingErrorMsg(code, 'LHSeigenName already exists.'), call. = FALSE)
+        }
         workEnv$LHSeigenName <- list(EigenName = EigenName, targetVar = targetVarProxy)
         workEnv[[EigenName]] <- TRUE
     } else { ## This is on the RHS
@@ -584,14 +768,13 @@ eigenize_nfVar <- function(code, symTab, typeEnv, workEnv) { ## A lot like eigen
     }
     code$eigMatrix <- TRUE
 
-    deparsedCode <- parse(text = nimDeparse(code), keep.source = FALSE)[[1]]
+    deparsedCode <- parse(text = nimDeparse(code), keep.source = FALSE)[[1]] 
     newExpr <- RparseTree2ExprClasses(as.name(EigenName))
     newExpr$type <- code$type
     newExpr$sizeExprs <- code$sizeExprs
     newExpr$nDim <- code$nDim
     newExpr$eigMatrix <- code$eigMatrix
     setArg(code$caller, code$callerArgID, newExpr)
-    
     if(!thisMapAlreadySet) {
         return(RparseTree2ExprClasses(
             EigenNewExpr(EigenName, deparsedCode, NULL, makeEigenTypeLabel(TRUE, code$type),
@@ -604,6 +787,7 @@ eigenize_nfVar <- function(code, symTab, typeEnv, workEnv) { ## A lot like eigen
 
 eigenizeName <- function(code, symTab, typeEnv, workEnv) {
     targetSym <- symTab$getSymbolObject(code$name, TRUE)
+    if(inherits(targetSym, 'symbolNimbleList')) return(NULL)
     if(!exists('nDim', envir = targetSym, inherits = FALSE)) {
        ## contextCode <- if(!is.null(code$caller)) paste(unlist(nimDeparse(code$caller)), collapse = '\n') else character()
         ## stop(paste0('in eigenizeName for ', nimDeparse(code), '. Symbol does not have an nDim.\n This occured within call\n', contextCode, collapse = ''), call. = FALSE)
@@ -633,7 +817,7 @@ eigenizeName <- function(code, symTab, typeEnv, workEnv) {
     if(!identical(targetSym$name, code$name)) { writeLines('found a case where !identical(targetSym$name, code$name)'); browser() }
     targetTypeSizeExprs <- code$sizeExprs
 
-    if(length(targetTypeSizeExprs) > 2) {stop(exprClassProcessingErrorMsg(code, 'Cannot eigenize a map of dimensions > 2.'), call. = FALSE)}
+    if(length(targetTypeSizeExprs) > 2) {stop(exprClassProcessingErrorMsg(code, 'Cannot do math with arrays that have more than 2 dimensions.'), call. = FALSE)}
     if(length(targetTypeSizeExprs) == 2) {
         nrowExpr <- targetTypeSizeExprs[[1]]
         ncolExpr <- targetTypeSizeExprs[[2]]
@@ -652,7 +836,10 @@ eigenizeName <- function(code, symTab, typeEnv, workEnv) {
 
     thisMapAlreadySet <- FALSE
     if(!is.null(workEnv[['OnLHSnow']])) { ## this is the var on the LHS
-        if(!is.null(workEnv[['LHSeigenName']])) stop(exprClassProcessingErrorMsg(code, 'LHSeigenName already exists.'), call. = FALSE) 
+        if(!is.null(workEnv[['LHSeigenName']])) {
+            browser()
+            stop(exprClassProcessingErrorMsg(code, 'LHSeigenName already exists.'), call. = FALSE)
+        }
         workEnv$LHSeigenName <- list(EigenName = EigenName, targetVar = code$name)
         workEnv[[EigenName]] <- TRUE
     } else { ## This is on the RHS
