@@ -13,6 +13,7 @@ if(!exists('.NimbleUseRegistration') || !.NimbleUseRegistration) {
    R_setPtrVectorOfPtrs = 'setPtrVectorOfPtrs'
    R_setOnePtrVectorOfPtrs = 'setOnePtrVectorOfPtrs'
    R_setDoublePtrFromSinglePtr = 'setDoublePtrFromSinglePtr'
+   R_setSmartPtrFromSinglePtr = 'setSmartPtrFromSinglePtr'
   ## R_setSinglePtrFromSinglePtr = 'setSinglePtrFromSinglePtr'
    C_dwish_chol = 'C_dwish_chol'
    C_rwish_chol = 'C_rwish_chol'
@@ -32,6 +33,10 @@ if(!exists('.NimbleUseRegistration') || !.NimbleUseRegistration) {
    C_rexp_nimble = 'C_rexp_nimble'
    C_pexp_nimble = 'C_pexp_nimble'
    C_qexp_nimble = 'C_qexp_nimble'
+   C_dinvgamma = 'C_dinvgamma'
+   C_rinvgamma = 'C_rinvgamma'
+   C_pinvgamma = 'C_pinvgamma'
+   C_qinvgamma = 'C_qinvgamma'
    C_dmnorm_chol = 'C_dmnorm_chol'
    C_rmnorm_chol = 'C_rmnorm_chol'
    C_dmvt_chol = 'C_dmvt_chol'
