@@ -4,6 +4,7 @@
 #include "nimble/dllFinalizer.h"
 //#include "nimble/RcppUtils.h"
 #include "nimble/Utils.h"
+#include "nimble/smartPtrs.h"
 #include<iostream>
 #include<sstream>
 #include<algorithm>
@@ -18,6 +19,21 @@ SEXP setDoublePtrFromSinglePtr(SEXP SdoublePtr, SEXP SsinglePtr) {
   return(R_NilValue);
 }
 
+// probably deprecated
+SEXP setSmartPtrFromSinglePtr(SEXP StoPtr, SEXP SfromPtr) {
+  void *fromPtr = R_ExternalPtrAddr(SfromPtr); 
+  nimSmartPtrBase *toSmartPtr = static_cast<nimSmartPtrBase*>(R_ExternalPtrAddr(StoPtr));  
+  toSmartPtr->setPtrFromVoidPtr(fromPtr);
+  return(R_NilValue);
+}
+
+SEXP setSmartPtrFromDoublePtr(SEXP StoPtr, SEXP SfromPtr) {
+  void *fromPtr = *static_cast<void**>(R_ExternalPtrAddr(SfromPtr)); 
+  nimSmartPtrBase *toSmartPtr = static_cast<nimSmartPtrBase*>(R_ExternalPtrAddr(StoPtr));  
+  toSmartPtr->setPtrFromVoidPtr(fromPtr);
+  return(R_NilValue);
+}
+
 SEXP setPtrVectorOfPtrs(SEXP SaccessorPtr, SEXP ScontentsPtr, SEXP Ssize) {
   vectorOfPtrsAccessBase *accessorPtr = static_cast<vectorOfPtrsAccessBase *>(R_ExternalPtrAddr(SaccessorPtr)); 
   void *contentsPtr = static_cast<void *>(R_ExternalPtrAddr(ScontentsPtr));
@@ -65,56 +81,6 @@ double t(double x) {return(x);}
 int prod(int x) {return(x);}
 double prod(double x) {return(x);}
 
-template<>
-void SEXP_2_NimArr<1>(SEXP Sn, NimArr<1, double> &ans) {
-  if(!(isNumeric(Sn) || isLogical(Sn))) PRINTF("Error: SEXP_2_NimArr<1> called for SEXP that is not a numeric or logical!\n");
-  int nn = LENGTH(Sn);
-  if(ans.size() != 0) PRINTF("Error: trying to reset a NimArr that was already sized\n");
-  ans.setSize(nn);
-  if(isReal(Sn)) {
-     std::copy(REAL(Sn), REAL(Sn) + nn, ans.getPtr());	
-  } else {
-    if(isInteger(Sn) || isLogical(Sn)) {
-      int *iSn = isInteger(Sn) ? INTEGER(Sn) : LOGICAL(Sn);
-      for(int i = 0; i < nn; ++i) {
-	ans(i) = static_cast<double>(iSn[i]);
-      }
-    } else {
-      PRINTF("Error: We could not handle the R input type to SEXP_2_NimArr<1>\n");
-    }
-  }
-}
-
-// Actually this is identical to above so could be done without specialization
-template<>
-void SEXP_2_NimArr<1>(SEXP Sn, NimArr<1, int> &ans) {
-  if(!(isNumeric(Sn) || isLogical(Sn))) PRINTF("Error: SEXP_2_NimArr<1> called for SEXP that is not a numeric or logical!\n");
-  int nn = LENGTH(Sn);
-  if(ans.size() != 0) PRINTF("Error: trying to reset a NimArr that was already sized\n");
-  ans.setSize(nn);
-  if(isReal(Sn)) {
-     std::copy(REAL(Sn), REAL(Sn) + nn, ans.getPtr());	
-  } else {
-    if(isInteger(Sn) || isLogical(Sn)) {
-      int *iSn = isInteger(Sn) ? INTEGER(Sn) : LOGICAL(Sn);
-      for(int i = 0; i < nn; ++i) {
-	ans(i) = static_cast<double>(iSn[i]);
-      }
-    } else {
-      PRINTF("Error: We could not handle the R input type to SEXP_2_NimArr<1>\n");
-    }
-  }
-}
-
-vector<int> getSEXPdims(SEXP Sx) {
-  if(!isNumeric(Sx)) {PRINTF("Error, getSEXPdims called for something not numeric\n"); return(vector<int>());}
-  if(!isVector(Sx)) {PRINTF("Error, getSEXPdims called for something not vector\n"); return(vector<int>());}
-  if(!isArray(Sx) & !isMatrix(Sx)) {
-    vector<int> ans; 
-    ans.resize(1); ans[0] = LENGTH(Sx); return(ans);
-  }
-  return(SEXP_2_vectorInt(getAttrib(Sx, R_DimSymbol), 0));
-}
 
 
 /* Cliff's new function for adding blank rows to C model values */
@@ -454,27 +420,27 @@ SEXP cGetMVElementOneRow(NimVecType* typePtr, nimType vecType, int index) 	{
 	return(R_NilValue);
 }
 
-  SEXP setMVElementFromList(SEXP vNimPtr, SEXP rList, SEXP Sindices){
-  	// This function needs to get the length of rList
-  	// Once it has this length, it marches down the length of rList and sets all the values of
-  	// of vNimPtr
-  	int listLength = LENGTH(rList);
-  	int checkLength = LENGTH(Sindices);
-  	if(listLength != checkLength){
-  		PRINTF("Number of indices copying to does not match length of list\n");
-  		return(R_NilValue);
-  	}
-	SEXP listElement;
-	NimVecType *typePtr = static_cast<NimVecType*>(R_ExternalPtrAddr(vNimPtr) );
-	nimType vecType = (*typePtr).getNimType();
-	int index;
-	for(int i = 0; i < listLength ; i++){
-		listElement = VECTOR_ELT(rList, i);
-		index = INTEGER(Sindices)[i];
-		cSetMVElementSingle(typePtr, vecType, index, listElement);
-	}
-  	return(R_NilValue);
+SEXP setMVElementFromList(SEXP vNimPtr, SEXP rList, SEXP Sindices){
+  // This function needs to get the length of rList
+  // Once it has this length, it marches down the length of rList and sets all the values of
+  // of vNimPtr
+  int listLength = LENGTH(rList);
+  int checkLength = LENGTH(Sindices);
+  if(listLength != checkLength){
+    PRINTF("Number of indices copying to does not match length of list\n");
+    return(R_NilValue);
   }
+  SEXP listElement;
+  NimVecType *typePtr = static_cast<NimVecType*>(R_ExternalPtrAddr(vNimPtr) );
+  nimType vecType = (*typePtr).getNimType();
+  int index;
+  for(int i = 0; i < listLength ; i++){
+    listElement = VECTOR_ELT(rList, i);
+    index = INTEGER(Sindices)[i];
+    cSetMVElementSingle(typePtr, vecType, index, listElement);
+  }
+  return(R_NilValue);
+}
 
 
  void cSetMVElementSingle(NimVecType* typePtr, nimType vecType,  int index, SEXP Svalue) {
@@ -562,7 +528,7 @@ SEXP setMVElement(SEXP Sextptr, SEXP Sindex, SEXP Svalue){
 	return(returnStatus(false) ) ;
   }
   cSetMVElementSingle( typePtr, vecType, index, Svalue );
-  SEXP SEXP_2_int(SEXP rPtr, SEXP refNum, SEXP rScalar);
+  //SEXP SEXP_2_int(SEXP rPtr, SEXP refNum, SEXP rScalar);
   return(returnStatus(true) );
  } 
  
@@ -640,6 +606,26 @@ SEXP NimArrInt_2_SEXP(NimArrBase<int> &nimArrInt){
 	return(Sans);
  }
 
+SEXP NimArrBool_2_SEXP(NimArrBase<bool> &nimArrBl){
+  int len = nimArrBl.size();
+  SEXP Sans;  
+  PROTECT(Sans = allocVector(LGLSXP, len));
+  //		std::copy(nimArrInt.v.begin(), nimArrInt.v.end() , INTEGER(Sans) );
+  std::copy(nimArrBl.v, nimArrBl.v + len , LOGICAL(Sans) );  
+  int numDims = nimArrBl.numDims();
+  if(numDims > 1) {
+    SEXP Sdim;
+    PROTECT(Sdim = allocVector(INTSXP, numDims));
+    for(int idim = 0; idim < numDims; ++idim) INTEGER(Sdim)[idim] = nimArrBl.dimSize(idim);
+    setAttrib(Sans, R_DimSymbol, Sdim);
+    UNPROTECT(2);
+  }
+  else {
+    UNPROTECT(1);
+  } 
+  return(Sans);
+ }
+
 
 NimArrType* getNimTypePtr(SEXP &rPtr, SEXP &refNum)
 {
@@ -668,7 +654,7 @@ void SEXP_2_NimArrDouble (SEXP rValues, NimArrBase<double> &NimArrDbl){
     for(int i = 0; i < rLength; i++)
       NimArrDbl[i] = REAL(rValues)[i];
   }
-  else if(isInteger(rValues) ) {
+  else if(isInteger(rValues) | isLogical(rValues) ) {
     for(int i = 0; i < rLength; i++)
       NimArrDbl[i] = INTEGER(rValues)[i];
   }
@@ -685,7 +671,7 @@ void SEXP_2_NimArrInt (SEXP rValues, NimArrBase<int> &NimArrInt){
     return;		
   }
   
-  if(isInteger(rValues) ) {
+  if(isInteger(rValues) | isLogical(rValues) ) {
     for(int i = 0; i < rLength; i++)
       NimArrInt[i] = INTEGER(rValues)[i];
   }
@@ -699,6 +685,29 @@ void SEXP_2_NimArrInt (SEXP rValues, NimArrBase<int> &NimArrInt){
   return;
 }
 
+void SEXP_2_NimArrBool (SEXP rValues, NimArrBase<bool> &NimArrBl){
+  int rLength = LENGTH(rValues);
+  if(rLength != NimArrBl.size() ) {
+    PRINTF("Warning: R object of different size than NimArrBl!\n");
+    return;		
+  }
+
+  // In R, Logical is represented as integer
+  if(isInteger(rValues) | isLogical(rValues)) {
+    for(int i = 0; i < rLength; i++)
+      NimArrBl[i] = INTEGER(rValues)[i];
+  }
+  else if(isReal(rValues) ) {
+    for(int i = 0; i < rLength; i++)
+      NimArrBl[i] = REAL(rValues)[i];
+  }
+  
+  else
+    PRINTF("WARNING: class of R object not recognized. Should be numeric or integer\n");    
+  return;
+}
+
+
 SEXP Nim_2_SEXP(SEXP rPtr, SEXP NumRefers){
 	NimArrType* nimTypePtr = getNimTypePtr(rPtr, NumRefers);
 	if(!nimTypePtr)
@@ -711,6 +720,11 @@ SEXP Nim_2_SEXP(SEXP rPtr, SEXP NumRefers){
 		NimArrBase<double>* nimBase = static_cast<NimArrBase<double> *>(nimTypePtr);
 		return(NimArrDouble_2_SEXP( (*nimBase) ) );
 	}
+	if(	(*nimTypePtr).getNimType() == BOOL){
+	  NimArrBase<bool>* nimBase = static_cast<NimArrBase<bool> *>(nimTypePtr);
+	  return(NimArrBool_2_SEXP( (*nimBase) ) );
+	}
+
 	PRINTF("Datatype of NimArr not found\n");
 	return(R_NilValue);
 }
@@ -723,33 +737,46 @@ SEXP SEXP_2_Nim(SEXP rPtr, SEXP NumRefers, SEXP rValues, SEXP allowResize){
 	NimArrType* nimTypePtr = getNimTypePtr(rPtr, NumRefers);
 	if(!nimTypePtr)
 		return(R_NilValue);
-	if(	(*nimTypePtr).getNimType() == INT){
-		NimArrBase<int>* nimBase = static_cast<NimArrBase<int> *>(nimTypePtr);
-        int nimNumDims = (*nimBase).numDims();
-        if(nimNumDims != sexpNumDims){
+	if((*nimTypePtr).getNimType() == INT){
+	  NimArrBase<int>* nimBase = static_cast<NimArrBase<int> *>(nimTypePtr);
+	  int nimNumDims = (*nimBase).numDims();
+	  if(nimNumDims != sexpNumDims){
             if((LENGTH(rValues) != (*nimBase).size()) & (sexpNumDims != 1)){
-                PRINTF("Incorrect number of dimensions in copying\n");
-                return(R_NilValue);
+	      PRINTF("Incorrect number of dimensions in copying\n");
+	      return(R_NilValue);
             }
-        }
-        if(resize == true)
+	  }
+	  if(resize == true)
             (*nimBase).setSize(sexpDims);
-		SEXP_2_NimArrInt(rValues,  (*nimBase) ) ;
+	  SEXP_2_NimArrInt(rValues,  (*nimBase) ) ;
 	}
-	if(	(*nimTypePtr).getNimType() == DOUBLE){
-		
-		NimArrBase<double>* nimBase = static_cast<NimArrBase<double> *>(nimTypePtr);
-        int nimNumDims = (*nimBase).numDims();
-                
-        if(nimNumDims != sexpNumDims){
+	if((*nimTypePtr).getNimType() == DOUBLE){
+	  
+	  NimArrBase<double>* nimBase = static_cast<NimArrBase<double> *>(nimTypePtr);
+	  int nimNumDims = (*nimBase).numDims();
+	  
+	  if(nimNumDims != sexpNumDims){
             if((LENGTH(rValues) != (*nimBase).size()) & (sexpNumDims != 1)){
-                PRINTF("Incorrect number of dimensions in copying\n");
-                return(R_NilValue);
+	      PRINTF("Incorrect number of dimensions in copying\n");
+	      return(R_NilValue);
             }
-        }
-        if((resize == true) & (nimNumDims == sexpNumDims))
+	  }
+	  if((resize == true) & (nimNumDims == sexpNumDims))
             (*nimBase).setSize(sexpDims);
-        SEXP_2_NimArrDouble( rValues, (*nimBase) ) ;
+	  SEXP_2_NimArrDouble( rValues, (*nimBase) ) ;
+	}
+	if((*nimTypePtr).getNimType() == BOOL){
+	  NimArrBase<bool>* nimBase = static_cast<NimArrBase<bool> *>(nimTypePtr);
+	  int nimNumDims = (*nimBase).numDims();
+	  if(nimNumDims != sexpNumDims){
+            if((LENGTH(rValues) != (*nimBase).size()) & (sexpNumDims != 1)){
+	      PRINTF("Incorrect number of dimensions in copying\n");
+	      return(R_NilValue);
+            }
+	  }
+	  if(resize == true)
+            (*nimBase).setSize(sexpDims);
+	  SEXP_2_NimArrBool(rValues,  (*nimBase) ) ;
 	}
 	return(R_NilValue);
 }
@@ -1105,20 +1132,3 @@ void nimble_optim_withVarArgs(void* nimFun, OptimControl* control, OptimAns* ans
 }	
 
 
-
-/* forwardsolve, backsolve */
-//
-//NimArr<1, double> forwardsolve(NimArr<2, double> A, NimArr<1, double> b) {
-//  NimArr<1, double> ans = NimArr<1, double>();
-//  return(ans);
-//}
-// 
-//NimArr<2, double> forwardsolve(NimArr<2, double> A, NimArr<2, double> B) {
-//  NimArr<2, double> ans = NimArr<2, double>();
-//  return(ans);
-//}
-
-
-
-
-
