@@ -41,7 +41,7 @@ NimArr<2, double> getBound_2D_double(int boundID, const oneNodeUseInfo &useInfo,
   return(useInfo.nodeFunPtr->getBound_2D_double_block(boundID, useInfo.useInfo));
 }
 
-
+// see include/nimble/nimbleEigenNimArr.h for some templated versions of calculate, simulate, calculateDiff and getLogProb used for arbitrary index vectors
 double calculate(NodeVectorClassNew &nodes) {
   double ans(0);
   const vector<oneNodeUseInfo> &useInfoVec = nodes.getUseInfoVec();
@@ -251,54 +251,24 @@ void ManyModelValuesMapAccessor::setRow(int i) {
 /////////
 // nimArr_2_[accessors]
 // nimArr is "from".  SMVAPtr is "to"
-template<class T>
-void nimArr_2_SingleModelAccess(SingleVariableMapAccessBase* SMVAPtr, NimArrBase<T> &nimArr, int nimBegin, int nimStride){
-  NimArrType* SMA_NimTypePtr = (*SMVAPtr).getNimArrPtr();
-  nimType SMA_Type = (*SMA_NimTypePtr).getNimType();
-  NimArrBase<double>* SMA_NimArrPtrD;
-  NimArrBase<int>* SMA_NimArrPtrI;
 
-  if(SMVAPtr->getSingleton()) {
-    switch(SMA_Type) {
-    case DOUBLE:
-      (*static_cast<NimArrBase<double> *>(SMA_NimTypePtr))[SMVAPtr->offset] = (*nimArr.getVptr())[nimBegin];
-      break;
-    case INT:
-      (*static_cast<NimArrBase<double> *>(SMA_NimTypePtr))[SMVAPtr->offset] = (*nimArr.getVptr())[nimBegin];
-      break;
-    default:
-      PRINTF("Copying type for nimArr_2_SingleModelAccess not supported\n");
-      break;
-    }
-  } else {
-    switch(SMA_Type) {
-    case DOUBLE:
-      SMA_NimArrPtrD = static_cast<NimArrBase<double>*>(SMA_NimTypePtr);
-      dynamicMapCopyFlatToDim<T, double>(SMA_NimArrPtrD, SMVAPtr->getOffset(), SMVAPtr->getStrides(), SMVAPtr->getSizes(), &nimArr, nimBegin, nimStride);
-      break;
-    case INT:
-      SMA_NimArrPtrI = static_cast<NimArrBase<int>*>(SMA_NimTypePtr);
-      dynamicMapCopyFlatToDim<T, int>(SMA_NimArrPtrI, SMVAPtr->getOffset(), SMVAPtr->getStrides(), SMVAPtr->getSizes(), &nimArr, nimBegin, nimStride);
-      break;
-    default:
-      PRINTF("Copying type for nimArr_2_SingleModelAccess not supported\n");
-      break;
-    }
-  }
-}
+// moved to accessorClasses.h so it can be seen from nimbleEigenNimArr.h
+// template<class T>
+// void nimArr_2_SingleModelAccess(SingleVariableMapAccessBase* SMVAPtr, NimArrBase<T> &nimArr, int nimBegin, int nimStride){
+
 
 template<class T>
 void nimArr_2_ManyModelAccess(ManyVariablesMapAccessor &MMVAPtr, NimArrBase<T> &nimArr){
-  vector<SingleVariableMapAccessBase*> SMVA_Vec = MMVAPtr.getMapAccessVector();
+  vector<SingleVariableMapAccessBase*> *SMVA_Vec = &(MMVAPtr.getMapAccessVector());
   int nimCurrent = 0;
   int nimEnd = nimArr.size();
   int nimArrStride = nimArr.strides()[0];
   int nimCurrentOffset = nimArr.getOffset();
-  int k = SMVA_Vec.size();
+  int k = SMVA_Vec->size();
   int nextNumVals;
   SingleVariableMapAccessBase* curSingleAccess;
   for(int i = 0; i < k ; i++){
-    curSingleAccess = SMVA_Vec[i];
+    curSingleAccess = (*SMVA_Vec)[i];
     nextNumVals = (*curSingleAccess).getLength();
     if(nextNumVals + nimCurrent > nimEnd){
       PRINTF("Warning: in nimArr_2_ManyModelAccess, accessor larger than NimArr!\n");
@@ -312,57 +282,46 @@ void nimArr_2_ManyModelAccess(ManyVariablesMapAccessor &MMVAPtr, NimArrBase<T> &
     PRINTF("Warning: after completing nimArr_2_ManyModelAccess, nimCurrent != nimEnd. Perhaps the NimArr was longer than the accessor?\n");
 }
 
-///////////////
-// [accessors]_2_nimArr
-// nimArr is "to". SMVAPtr is "from"
 template<class T>
-void SingleModelAccess_2_nimArr(SingleVariableMapAccessBase* SMVAPtr, NimArrBase<T> &nimArr, int nimBegin, int nimStride){
-  NimArrType* SMA_NimTypePtr = (*SMVAPtr).getNimArrPtr();
-  nimType SMA_Type = (*SMA_NimTypePtr).getNimType();
-  NimArrBase<double>* SMA_NimArrPtrD;
-  NimArrBase<int>* SMA_NimArrPtrI;
-  if(SMVAPtr->getSingleton()) {
-    switch(SMA_Type) {
-    case DOUBLE:
-      (*nimArr.getVptr())[nimBegin] = (*static_cast<NimArrBase<double> *>(SMA_NimTypePtr))[SMVAPtr->offset];
-      break;
-    case INT:
-      (*nimArr.getVptr())[nimBegin] = (*static_cast<NimArrBase<int> *>(SMA_NimTypePtr))[SMVAPtr->offset];
-      break;
-    default:
-      PRINTF("Copying type for SingleModelAccess_2_nimArr not supported\n");
-      break;
-    }
-  } else {
-    switch(SMA_Type) {
-    case DOUBLE:
-      SMA_NimArrPtrD = static_cast<NimArrBase<double>*>(SMA_NimTypePtr);
-      dynamicMapCopyDimToFlat<double, T>(&nimArr, nimBegin, nimStride, SMA_NimArrPtrD, SMVAPtr->getOffset(), SMVAPtr->getStrides(), SMVAPtr->getSizes() );
-      break;
-    case INT:
-      SMA_NimArrPtrI = static_cast<NimArrBase<int>*>(SMA_NimTypePtr);
-      dynamicMapCopyDimToFlat<int, T>(&nimArr, nimBegin, nimStride, SMA_NimArrPtrI, SMVAPtr->getOffset(), SMVAPtr->getStrides(), SMVAPtr->getSizes());
-      break;
-    default:
-      PRINTF("Copying type for SingleModelAccess_2_nimArr not supported\n");
-      break;
-    }
-  }
-}
-
-
-template<class T>
-void ManyModelAccess_2_nimArr(ManyVariablesMapAccessor &MMVAPtr, NimArrBase<T> &nimArr){
-  vector<SingleVariableMapAccessBase*> SMVA_Vec = MMVAPtr.getMapAccessVector();
+void nimArr_2_ManyModelAccessIndex(ManyVariablesMapAccessor &MMVAPtr, NimArrBase<T> &nimArr, int index){
+  vector<SingleVariableMapAccessBase*> *SMVA_Vec = &(MMVAPtr.getMapAccessVector());
   int nimCurrent = 0;
   int nimEnd = nimArr.size();
   int nimArrStride = nimArr.strides()[0];
   int nimCurrentOffset = nimArr.getOffset();
-  int k = SMVA_Vec.size();
+  int nextNumVals;
+  SingleVariableMapAccessBase* curSingleAccess;
+  curSingleAccess = (*SMVA_Vec)[index];
+  nextNumVals = (*curSingleAccess).getLength();
+  if(nextNumVals + nimCurrent > nimEnd)
+    PRINTF("Warning: in nimArr_2_ManyModelAccessIndex, accessor larger than NimArr!\n");
+  nimArr_2_SingleModelAccess<T>(curSingleAccess, nimArr, nimCurrentOffset, nimArrStride);
+  nimCurrent += nextNumVals;
+  if(nimCurrent != nimEnd)
+    PRINTF("Warning: after completing nimArr_2_ManyModelAccessIndex, nimCurrent != nimEnd. Perhaps the NimArr was longer than the accessor?\n");
+}
+  
+///////////////
+// [accessors]_2_nimArr
+// nimArr is "to". SMVAPtr is "from"
+
+// Moved to accessorClasses.h so nimbleEigenNimArr.h can see it
+//template<class T>
+//void SingleModelAccess_2_nimArr(SingleVariableMapAccessBase* SMVAPtr, NimArrBase<T> &nimArr, int nimBegin, int nimStride){
+
+
+template<class T>
+void ManyModelAccess_2_nimArr(ManyVariablesMapAccessor &MMVAPtr, NimArrBase<T> &nimArr){
+  vector<SingleVariableMapAccessBase*> *SMVA_Vec = &(MMVAPtr.getMapAccessVector());
+  int nimCurrent = 0;
+  int nimEnd = nimArr.size();
+  int nimArrStride = nimArr.strides()[0];
+  int nimCurrentOffset = nimArr.getOffset();
+  int k = SMVA_Vec->size();
   int nextNumVals;
   SingleVariableMapAccessBase* curSingleAccess;
   for(int i = 0; i < k ; i++){
-    curSingleAccess = SMVA_Vec[i];
+    curSingleAccess = (*SMVA_Vec)[i];
     nextNumVals = (*curSingleAccess).getLength();
     if(nextNumVals + nimCurrent > nimEnd){
       PRINTF("Warning: in nimArr_2_ManyModelAccess, accessor larger than NimArr!\n");
@@ -376,6 +335,24 @@ void ManyModelAccess_2_nimArr(ManyVariablesMapAccessor &MMVAPtr, NimArrBase<T> &
     PRINTF("Warning: after completing ManyModelAccess_2_nimArr, nimCurrent != nimEnd. Perhaps the NimArr was longer than the accessor?\n");
 }
 
+template<class T>
+void ManyModelAccessIndex_2_nimArr(ManyVariablesMapAccessor &MMVAPtr, NimArrBase<T> &nimArr, int index) {
+  vector<SingleVariableMapAccessBase*> *SMVA_Vec = &(MMVAPtr.getMapAccessVector());
+  int nimCurrent = 0;
+  int nimEnd = nimArr.size();
+  int nimArrStride = nimArr.strides()[0];
+  int nimCurrentOffset = nimArr.getOffset();
+  int nextNumVals;
+  SingleVariableMapAccessBase* curSingleAccess;
+  curSingleAccess = (*SMVA_Vec)[index];
+  nextNumVals = (*curSingleAccess).getLength();
+  if(nextNumVals + nimCurrent > nimEnd)
+    PRINTF("Warning: in ManyModelAccessIndex_2_nimArr, accessor larger than NimArr!\n");
+  SingleModelAccess_2_nimArr<T>(curSingleAccess, nimArr, nimCurrentOffset, nimArrStride);
+  //SingleModelAccess_2_nimArr<T>(curSingleAccess, nimArr, 0, 0 ); 
+}
+
+
 //////////
 //
 void setValues(NimArrBase<double> &nimArr, ManyVariablesMapAccessor &MVA){
@@ -386,6 +363,14 @@ void setValues(NimArrBase<int> &nimArr, ManyVariablesMapAccessor &MVA){
 	nimArr_2_ManyModelAccess<int>(MVA, nimArr);
 }
 
+void setValues(NimArrBase<double> &nimArr, ManyVariablesMapAccessor &MVA, int index){
+  nimArr_2_ManyModelAccessIndex<double>(MVA, nimArr, index-1);
+}
+
+void setValues(NimArrBase<int> &nimArr, ManyVariablesMapAccessor &MVA, int index){
+  nimArr_2_ManyModelAccessIndex<int>(MVA, nimArr, index-1);
+}
+
 void getValues(NimArr<1, double> &nimArr, ManyVariablesMapAccessor &MVA){
 	ManyModelAccess_2_nimArr<double>(MVA, nimArr);
 }
@@ -393,6 +378,14 @@ void getValues(NimArr<1, int> &nimArr, ManyVariablesMapAccessor &MVA){
 	ManyModelAccess_2_nimArr<int>(MVA, nimArr);
 }
 
+void getValues(NimArr<1, double> &nimArr, ManyVariablesMapAccessor &MVA, int index){
+  ManyModelAccessIndex_2_nimArr<double>(MVA, nimArr, index-1);
+  } 
+
+void getValues(NimArr<1, int> &nimArr, ManyVariablesMapAccessor &MVA, int index){
+  ManyModelAccessIndex_2_nimArr<int>(MVA, nimArr, index-1);
+}
+
 ////////////////////////
 // Old versions
 // Functions for copying
@@ -426,14 +419,14 @@ void nimArr_2_SingleModelAccess(SingleVariableAccessBase* SMVAPtr, NimArrBase<T>
 //void nimArr_2_ManyModelAccess(ManyVariablesAccessor &MMVAPtr, NimArr<D, T> &nimArr){
 template<class T>
 void nimArr_2_ManyModelAccess(ManyVariablesAccessor &MMVAPtr, NimArrBase<T> &nimArr) {
-  vector<SingleVariableAccessBase*> SMVA_Vec = MMVAPtr.getAccessVector();
+  vector<SingleVariableAccessBase*> *SMVA_Vec = &(MMVAPtr.getAccessVector());
   int nimCurrent = 0;
   int nimEnd = nimArr.size();
-  int k = SMVA_Vec.size();
+  int k = SMVA_Vec->size();
   int nextNumVals;
   SingleVariableAccessBase* curSingleAccess;
   for(int i = 0; i < k ; i++){
-    curSingleAccess = SMVA_Vec[i];
+    curSingleAccess = (*SMVA_Vec)[i];
     nextNumVals = (*curSingleAccess).getLength();
     if(nextNumVals + nimCurrent > nimEnd){
       PRINTF("Warning: in nimArr_2_ManyModelAccess, accessor larger than NimArr!\n");
@@ -471,27 +464,26 @@ void SingleModelAccess_2_nimArr(SingleVariableAccessBase* SMVAPtr, NimArr<D, T>
 
 template<int D, class T>
 void ManyModelAccess_2_nimArr(ManyVariablesAccessor &MMVAPtr, NimArr<D, T> &nimArr) {
-  vector<SingleVariableAccessBase*> SMVA_Vec = MMVAPtr.getAccessVector();
+  vector<SingleVariableAccessBase*> *SMVA_Vec = &(MMVAPtr.getAccessVector());
   int nimCurrent = 0;
   int nimEnd = nimArr.size();
-  int k = SMVA_Vec.size();
+  int k = SMVA_Vec->size();
   int nextNumVals;
   SingleVariableAccessBase* curSingleAccess;
   for(int i = 0; i < k ; i++){
-    curSingleAccess = SMVA_Vec[i];
+    curSingleAccess = (*SMVA_Vec)[i];
     nextNumVals = (*curSingleAccess).getLength();
     if(nextNumVals + nimCurrent > nimEnd){
-      PRINTF("Warning: in nimArr_2_ManyModelAccess, accessor larger than NimArr!\n");
+      PRINTF("Warning: in ManyModelAccess_2_nimArr, accessor larger than NimArr!\n");
       break;
     }
     SingleModelAccess_2_nimArr<D, T>(curSingleAccess, nimArr, nimCurrent);
     nimCurrent = nimCurrent + nextNumVals;
   }
   if(nimCurrent != nimEnd)
-    PRINTF("Warning: after completing nimArr_2_ManyModelAccess, nimCurrent != nimEnd. Perhaps the NimArr was longer than the accessor?\n");
+    PRINTF("Warning: after completing ManyModelAccess_2_nimArr, nimCurrent != nimEnd. Perhaps the NimArr was longer than the accessor?\n");
 }
 
-
 void setValues(NimArrBase<double> &nimArr, ManyVariablesAccessor &MVA){
 	nimArr_2_ManyModelAccess<double>(MVA, nimArr);
 }
@@ -506,6 +498,7 @@ void getValues(NimArr<1, int> &nimArr, ManyVariablesAccessor &MVA){
 	ManyModelAccess_2_nimArr<1, int>(MVA, nimArr);
 }
 
+
 // new copierClass versions
 // remember to look at calculate() too to avoid copies every time.
 void nimCopy(const copierVectorClass &copiers) {
@@ -541,8 +534,8 @@ void nimCopy(copierVectorClass &copiers, int rowFrom, int rowTo, int unused) { /
 
 void copierVectorClass::setup(ManyVariablesMapAccessorBase *from, ManyVariablesMapAccessorBase *to, int isFromMV, int isToMV) {
   // Imitates old version of nimCopy but populates copyVector of correct choices of derived copierClass objects
-  vector<SingleVariableMapAccessBase *> fromAccessors = from->getMapAccessVector();
-  vector<SingleVariableMapAccessBase *> toAccessors = to->getMapAccessVector();
+  vector<SingleVariableMapAccessBase *> *fromAccessors = &(from->getMapAccessVector());
+  vector<SingleVariableMapAccessBase *> *toAccessors = &(to->getMapAccessVector());
 
 #ifdef __NIMBLE_DEBUG_ACCESSORS
   PRINTF("Entering nimCopy\n");
@@ -550,17 +543,17 @@ void copierVectorClass::setup(ManyVariablesMapAccessorBase *from, ManyVariablesM
   to->check();
 #endif
 
-  if(fromAccessors.size() != toAccessors.size()) {
-    _nimble_global_output<<"Error in setting up a copierVector: from and to access vectors have sizes "<<fromAccessors.size() << " and " << toAccessors.size() << "\n";
+  if(fromAccessors->size() != toAccessors->size()) {
+    _nimble_global_output<<"Error in setting up a copierVector: from and to access vectors have sizes "<<fromAccessors->size() << " and " << toAccessors->size() << "\n";
     nimble_print_to_R(_nimble_global_output);
   }
-  copyVector.resize( fromAccessors.size() );
+  copyVector.resize( fromAccessors->size() );
   //PRINTF("Ready to set up length %i\n", copyVector.size());
   vector<SingleVariableMapAccessBase *>::iterator iFrom, iTo, iFromEnd;
-  iFromEnd = fromAccessors.end();
-  iTo =  toAccessors.begin();
+  iFromEnd = fromAccessors->end();
+  iTo =  toAccessors->begin();
   int i = 0;
-  for(iFrom = fromAccessors.begin(); iFrom != iFromEnd; iFrom++) {
+  for(iFrom = fromAccessors->begin(); iFrom != iFromEnd; iFrom++) {
     //PRINTF("setting up %i\n", i);
     copyVector[i] = makeOneCopyClass(*iFrom, *iTo, isFromMV, isToMV); // switched from isFromMV and isToMV
     iTo++;
@@ -580,24 +573,24 @@ copierVectorClass::~copierVectorClass() {
 // new MapAccessor versions
 void nimCopy(ManyVariablesMapAccessorBase &from, ManyVariablesMapAccessorBase &to) { //map version
 
-  vector<SingleVariableMapAccessBase *> fromAccessors = from.getMapAccessVector();
-  vector<SingleVariableMapAccessBase *> toAccessors = to.getMapAccessVector();
+  vector<SingleVariableMapAccessBase *> *fromAccessors = &(from.getMapAccessVector());
+  vector<SingleVariableMapAccessBase *> *toAccessors = &(to.getMapAccessVector());
 
 #ifdef __NIMBLE_DEBUG_ACCESSORS
   PRINTF("Entering nimCopy\n");
-  from.check();
-  to.check();
+  from->check();
+  to->check();
 #endif
 
-  if(fromAccessors.size() != toAccessors.size()) {
-    _nimble_global_output<<"Error in nimCopy: from and to access vectors have sizes "<<fromAccessors.size() << " and " << toAccessors.size() << "\n";
+  if(fromAccessors->size() != toAccessors->size()) {
+    _nimble_global_output<<"Error in nimCopy: from and to access vectors have sizes "<<fromAccessors->size() << " and " << toAccessors->size() << "\n";
     nimble_print_to_R(_nimble_global_output);
   }
 
   vector<SingleVariableMapAccessBase *>::iterator iFrom, iTo, iFromEnd;
-  iFrom = fromAccessors.begin();
-  iFromEnd = fromAccessors.end();
-  iTo =  toAccessors.begin();
+  iFrom = fromAccessors->begin();
+  iFromEnd = fromAccessors->end();
+  iTo =  toAccessors->begin();
   for( ; iFrom != iFromEnd; ++iFrom) {
     nimCopyOne(*iFrom, *iTo);
     ++iTo;
@@ -847,18 +840,18 @@ void dynamicMapCopyCheck(NimArrType *NAT, int offset, vector<int> &strides, vect
 // old versions:
 void nimCopy(ManyVariablesAccessorBase &from, ManyVariablesAccessorBase &to) {
 
-  vector<SingleVariableAccessBase *> fromAccessors = from.getAccessVector();
-  vector<SingleVariableAccessBase *> toAccessors = to.getAccessVector();
+  vector<SingleVariableAccessBase *> *fromAccessors = &(from.getAccessVector());
+  vector<SingleVariableAccessBase *> *toAccessors = &(to.getAccessVector());
 
-  if(fromAccessors.size() != toAccessors.size()) {
-    _nimble_global_output<<"Error in nimCopy: from and to access vectors have sizes "<<fromAccessors.size() << " and " << toAccessors.size() << "\n";
+  if(fromAccessors->size() != toAccessors->size()) {
+    _nimble_global_output<<"Error in nimCopy: from and to access vectors have sizes "<<fromAccessors->size() << " and " << toAccessors->size() << "\n";
     nimble_print_to_R(_nimble_global_output);
   }
 
   vector<SingleVariableAccessBase *>::iterator iFrom, iTo, iFromEnd;
-  iFrom = fromAccessors.begin();
-  iFromEnd = fromAccessors.end();
-  iTo =  toAccessors.begin();
+  iFrom = fromAccessors->begin();
+  iFromEnd = fromAccessors->end();
+  iTo =  toAccessors->begin();
   for( ; iFrom != iFromEnd; ++iFrom) {
     nimCopyOne(*iFrom, *iTo);
     ++iTo;
@@ -1239,20 +1232,21 @@ SEXP populateIndexedNodeInfoTable(SEXP StablePtr, SEXP StableContents) {
     if(ncol != 0) {PRINTF("Warning from populateIndexedNodeInfoTable: nrow == 0 but ncol != 0.");}
     UNPROTECT(1);
     return(R_NilValue);
-  }
-
-  if(!isNumeric(StableContents)) {PRINTF("Warning from populateIndexedNodeInfoTable: StableContents is not numeric"); return(R_NilValue);}
-  if(isInteger(StableContents)) {
-    int *contentsPtr = INTEGER(StableContents);
-    tablePtr->reserve(nrow);
-    for(int i = 0; i < nrow; i++) {
-      tablePtr->push_back(indexedNodeInfo(contentsPtr + i, ncol, nrow));
-    }
   } else {
-    double *contentsPtrd = REAL(StableContents);
-    tablePtr->reserve(nrow);
-    for(int i = 0; i < nrow; i++) {
-      tablePtr->push_back(indexedNodeInfo(contentsPtrd + i, ncol, nrow));
+
+    if(!isNumeric(StableContents)) {PRINTF("Warning from populateIndexedNodeInfoTable: StableContents is not numeric"); return(R_NilValue);}
+    if(isInteger(StableContents)) {
+      int *contentsPtr = INTEGER(StableContents);
+      tablePtr->reserve(nrow);
+      for(int i = 0; i < nrow; i++) {
+	tablePtr->push_back(indexedNodeInfo(contentsPtr + i, ncol, nrow));
+      }
+    } else {
+      double *contentsPtrd = REAL(StableContents);
+      tablePtr->reserve(nrow);
+      for(int i = 0; i < nrow; i++) {
+	tablePtr->push_back(indexedNodeInfo(contentsPtrd + i, ncol, nrow));
+      }
     }
   }
   //  std::cout<<"done with size "<<tablePtr->size()<<"\n";
