@@ -11,6 +11,13 @@ using std::string;
 
 //using namespace std;
 
+// A utility function that will return floor(x) unless x is within numerical imprecision of an integer, in which case it will return round(x)
+int floorOrEquivalent(double x);
+
+int rFunLength(int Arg);
+int rFunLength(double Arg);
+int rFunLength(bool Arg);
+
 class nimbleTimerClass_ {
  public:
   clock_t t_start;
@@ -110,6 +117,8 @@ bool decide(double lMHr);
 void nimStop(string msg);
 void nimStop();
 
+bool nimNot(bool x);
+
 // needed for link functions
 double ilogit(double x);
 double icloglog(double x);
