@@ -10,6 +10,7 @@
 #include <nimble/dists.h>
 #include <nimble/NamedObjects.h>
 #include <nimble/dllFinalizer.h>
+#include <nimble/smartPtrs.h>
 
 #include <R_ext/Rdynload.h>
 
@@ -66,6 +67,8 @@ R_CallMethodDef CallEntries[] = {
 
   //RcppNimbleUtils
   FUN(setDoublePtrFromSinglePtr, 2),
+  FUN(setSmartPtrFromSinglePtr, 2),
+  FUN(setSmartPtrFromDoublePtr, 2),
   FUN(addBlankModelValueRows, 2),
   FUN(getNRow, 1),
   FUN(copyModelValuesElements, 4),
@@ -82,6 +85,18 @@ R_CallMethodDef CallEntries[] = {
   FUN(makeNumericList, 3),
   FUN(Nim_2_SEXP, 2),
   FUN(SEXP_2_Nim, 4),
+  FUN(extract_double_2_SEXP, 2),
+  FUN(populate_SEXP_2_double, 3),
+  FUN(extract_int_2_SEXP, 2),
+  FUN(populate_SEXP_2_int, 3),
+  FUN(extract_bool_2_SEXP, 2),
+  FUN(populate_SEXP_2_bool, 3),
+  
+  FUN(populate_SEXP_2_string, 2),
+  FUN(extract_string_2_SEXP, 1),
+  FUN(populate_SEXP_2_stringVector, 2),
+  FUN(extract_stringVector_2_SEXP, 1),
+
   FUN(setPtrVectorOfPtrs, 3),
   FUN(setOnePtrVectorOfPtrs, 3),
   //FUN(getOnePtrVectorOfPtrs, 2),
@@ -101,7 +116,8 @@ R_CallMethodDef CallEntries[] = {
   FUN(register_namedObjects_Finalizer, 3),
   FUN(register_numberedObjects_Finalizer, 3),
   FUN(register_VecNimArr_Finalizer, 2),
-
+  FUN(register_pointedToBase_Finalizer, 3),
+  FUN(register_smartPtrBase_Finalizer, 3),
   FUN(RNimble_Ptr_ManualFinalizer, 1),
   FUN(RNimble_Ptr_CheckAndRunAllDllFinalizers, 2),
   FUN(CountDllObjects, 1),
