@@ -1,7 +1,11 @@
+#include <Eigen/Dense>
+#include <iostream> // must go before other things because R defines a "length" macro
 #include <nimble/RcppUtils.h>
+#include "nimble/RcppNimbleUtils.h"
 //#include <nimble/ModelClassUtils.h>
 //#include <nimble/accessorClasses.h>
 #include <nimble/nimbleGraph.h>
+#include <nimble/EigenTypedefs.h>
 #include <nimble/dists.h>
 
 #include <R_ext/Rdynload.h>
@@ -35,10 +39,25 @@ R_CallMethodDef CallEntries[] = {
  FUN(C_rcat, 2),
  FUN(C_dt_nonstandard, 5),
  FUN(C_rt_nonstandard, 4),
+ FUN(C_qt_nonstandard, 6),
+ FUN(C_pt_nonstandard, 6),
  FUN(C_dmnorm_chol, 5),
  FUN(C_rmnorm_chol, 3),
  FUN(C_dinterval, 4),
  FUN(C_rinterval, 3),
+ FUN(C_dmvt_chol, 6),
+ FUN(C_rmvt_chol, 4),
+ FUN(C_dexp_nimble, 3),
+ FUN(C_rexp_nimble, 2),
+ FUN(C_pexp_nimble, 4),
+ FUN(C_qexp_nimble, 4),
+ FUN(C_dinvgamma, 4),
+ FUN(C_rinvgamma, 3),
+ FUN(C_pinvgamma, 5),
+ FUN(C_qinvgamma, 5),
+ FUN(C_nimEigen, 3),
+ FUN(C_nimSvd, 3),
+
  // FUN(makeNumericList, 3),
 //   The following 4 conflict with names of R functions. So we prefix them with a R_
  //CFUN(setPtrVectorOfPtrs, 3),
@@ -61,14 +80,14 @@ R_CallMethodDef CallEntries[] = {
  FUN(fastMatrixInsert, 4),
  FUN(matrix2ListDouble, 5),
  FUN(matrix2ListInt, 5),
- // FUN(rankSample, 4),
+ FUN(C_rankSample, 4),
  FUN(parseVar, 1),
 
  FUN(setGraph, 6),
  FUN(anyStochDependencies, 1),
  FUN(anyStochParents, 1),
  FUN(getDependencies, 4),
- 
+ FUN(getDependencyPathCountOneNode, 2), 
  {NULL, NULL, 0}
 };
 
@@ -80,6 +99,7 @@ void
 R_init_nimble(DllInfo *dll)
 {
     R_registerRoutines(dll, NULL, CallEntries, NULL, NULL);
+    R_useDynamicSymbols(dll, FALSE);
 }
 
 
